@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * History:
  * --------
@@ -66,9 +66,6 @@
 #define EVENT_RT_NO RT_NO 		/*!< event_route routing tables number */
 #define DEFAULT_RT 0 			/*!< default routing table */
 
-#define MAX_REC_LEV 100 		/*!< maximum number of recursive calls */
-#define ROUTE_MAX_REC_LEV 100 		/*!< maximum number of recursive calls for route()*/
-
 #define MAX_URI_SIZE 1024		/*!< Max URI size used when rewriting URIs */
 
 #define MAX_PATH_SIZE 256 		/*!< Maximum length of path header buffer */
@@ -147,11 +144,11 @@
 #define SRV_MAX_PREFIX_LEN SRV_TLS_PREFIX_LEN
 
 #ifndef PKG_MEM_SIZE
-#define PKG_MEM_SIZE 4
+#define PKG_MEM_SIZE 8
 #endif
 #define PKG_MEM_POOL_SIZE PKG_MEM_SIZE*1024*1024	/*!< used only if PKG_MALLOC is defined*/
 
-#define SHM_MEM_SIZE 32				/*!< used if SH_MEM is defined*/
+#define SHM_MEM_SIZE 64				/*!< used if SH_MEM is defined*/
 
 
 /* dimensioning buckets in q_malloc */
@@ -169,7 +166,7 @@
 #define BUF_SIZE 65535
 
 #define MAX_VIA_LINE_SIZE	240	/*!< forwarding  -- Via buffer dimensioning */
-#define MAX_RECEIVED_SIZE	57	/*!< forwarding  -- Via buffer dimensioning - Received header */
+#define MAX_RECEIVED_SIZE	59	/*!< forwarding  -- Via buffer dimensioning - Received header */
 #define MAX_RPORT_SIZE		13	/*!< forwarding  -- Via buffer dimensioning - Rport */
 
 #define MAX_BRANCHES    	12	/*!< maximum number of branches per transaction */
