@@ -22,7 +22,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * History:
  * -------
@@ -121,6 +121,12 @@
 static int mhomed_sock_cache_disabled = 0;
 static int sock_inet = -1;
 static int sock_inet6 = -1;
+static int _forward_set_send_info = 0;
+
+void forward_set_send_info(int v)
+{
+	_forward_set_send_info = v;
+}
 
 static void apply_force_send_socket(struct dest_info* dst, struct sip_msg* msg);
 
@@ -137,7 +143,7 @@ struct socket_info* get_out_socket(union sockaddr_union* to, int proto)
 	uncon.sin.sin_family = AF_UNSPEC;
 
 	if (unlikely(proto!=PROTO_UDP)) {
-		LOG(L_CRIT, "BUG: get_out_socket can only be called for UDP\n");
+		LM_CRIT("can only be called for UDP\n");
 		return 0;
 	}
 retry:
@@ -191,14 +197,12 @@ retry:
 			}
 			goto retry;
 		}
-		LOG(L_ERR, "ERROR: get_out_socket: connect failed: %s\n",
-				strerror(errno));
+		LM_ERR("connect failed: %s\n", strerror(errno));
 		goto error;
 	}
 	len=sizeof(from);
 	if (unlikely(getsockname(*temp_sock, &from.s, &len)==-1)) {
-		LOG(L_ERR, "ERROR: get_out_socket: getsockname failed: %s\n",
-				strerror(errno));
+		LM_ERR("getsockname failed: %s\n", strerror(errno));
 		goto error;
 	}
 	su2ip_addr(&ip, &from);
@@ -211,7 +215,9 @@ retry:
 	}
 	return si;
 error:
-	LOG(L_ERR, "ERROR: get_out_socket: no socket found\n");
+	LM_ERR("no socket found\n");
+	ERR("no corresponding socket found for(%s:%s)\n",
+			proto2a(proto), su2a(to, sizeof(*to)));
 	if (unlikely(mhomed_sock_cache_disabled && *temp_sock >=0)){
 		close(*temp_sock);
 		*temp_sock=-1;
@@ -256,9 +262,7 @@ struct socket_info* get_send_socket2(struct socket_info* force_send_socket,
 											proto);
 			if (unlikely(force_send_socket == 0)){
 				if (likely(mismatch)) *mismatch=SS_MISMATCH_ADDR;
-				LOG(L_WARN, "WARNING: get_send_socket: "
-						"protocol/port mismatch (forced %s:%s:%d,"
-						" to %s:%s)\n",
+				LM_WARN("protocol/port mismatch (forced %s:%s:%d, to %s:%s)\n",
 						proto2a(orig->proto), ip_addr2a(&orig->address),
 						orig->port_no,
 						proto2a(proto), su2a(to, sizeof(*to)));
@@ -289,8 +293,7 @@ struct socket_info* get_send_socket2(struct socket_info* force_send_socket,
 				return force_send_socket;
 		else{
 			if (!(force_send_socket->flags & SI_IS_MCAST))
-				LOG(L_WARN, "WARNING: get_send_socket: not listening"
-							 " on the requested socket (%s:%s:%d),"
+				LM_WARN("not listening on the requested socket (%s:%s:%d),"
 							 " no fork mode?\n",
 							proto2a(force_send_socket->proto),
 							ip_addr2a(&force_send_socket->address),
@@ -304,7 +307,7 @@ not_forced:
 		if ((send_sock==0) || (send_sock->socket!=-1))
 			return send_sock; /* found or error*/
 		else if (send_sock->socket==-1){
-			LOG(L_WARN, "WARNING: get_send_socket: not listening on the"
+			LM_WARN("not listening on the"
 					" requested socket (%s:%s:%d), no fork mode?\n",
 					proto2a(send_sock->proto), ip_addr2a(&send_sock->address),
 					send_sock->port_no);
@@ -328,8 +331,8 @@ not_forced:
 								break;
 				case AF_INET6:	send_sock=sendipv6_tcp;
 								break;
-				default:	LOG(L_ERR, "get_send_socket: BUG: don't know how"
-									" to forward to af %d\n", to->s.sa_family);
+				default:	LM_ERR("don't know how to forward to af %d\n",
+									to->s.sa_family);
 			}
 			break;
 #endif
@@ -342,8 +345,8 @@ not_forced:
 								break;
 				case AF_INET6:	send_sock=sendipv6_tls;
 								break;
-				default:	LOG(L_ERR, "get_send_socket: BUG: don't know how"
-									" to forward to af %d\n", to->s.sa_family);
+				default:	LM_ERR("don't know how to forward to af %d\n",
+									to->s.sa_family);
 			}
 			break;
 #endif /* USE_TLS */
@@ -357,8 +360,7 @@ not_forced:
 									break;
 					case AF_INET6:	send_sock=sendipv6_sctp;
 									break;
-					default:	LOG(L_ERR, "get_send_socket: BUG: don't know"
-										" how to forward to af %d\n",
+					default:	LM_ERR("don't know how to forward to af %d\n",
 										to->s.sa_family);
 				}
 			}else send_sock=bind_address;
@@ -373,15 +375,13 @@ not_forced:
 									break;
 					case AF_INET6:	send_sock=sendipv6;
 									break;
-					default:	LOG(L_ERR, "get_send_socket: BUG: don't know"
-										" how to forward to af %d\n",
+					default:	LM_ERR("don't know how to forward to af %d\n",
 										to->s.sa_family);
 				}
 			}else send_sock=bind_address;
 			break;
 		default:
-			LOG(L_CRIT, "BUG: get_send_socket: unsupported proto %d (%s)\n",
-					proto, proto2a(proto));
+			LM_CRIT("unsupported proto %d (%s)\n", proto, proto2a(proto));
 	}
 	return send_sock;
 }
@@ -497,6 +497,7 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 	int ret;
 	struct ip_addr ip; /* debugging only */
 	char proto;
+	struct onsend_info onsnd_info = {0};
 #ifdef USE_DNS_FAILOVER
 	struct socket_info* prev_send_sock;
 	int err;
@@ -519,17 +520,15 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 			err=dns_sip_resolve2su(&dns_srv_h, &send_info->to, dst, port,
 									&proto, dns_flags);
 			if (err!=0){
-				LOG(L_ERR, "ERROR: forward_request: resolving \"%.*s\""
-						" failed: %s [%d]\n", dst->len, ZSW(dst->s),
-						dns_strerror(err), err);
+				LM_ERR("resolving \"%.*s\" failed: %s [%d]\n",
+						dst->len, ZSW(dst->s), dns_strerror(err), err);
 				ret=E_BAD_ADDRESS;
 				goto error;
 			}
 		}else
 #endif
 		if (sip_hostport2su(&send_info->to, dst, port, &proto)<0){
-			LOG(L_ERR, "ERROR: forward_request: bad host name %.*s,"
-						" dropping packet\n", dst->len, ZSW(dst->s));
+			LM_ERR("bad host name %.*s, dropping packet\n", dst->len, ZSW(dst->s));
 			ret=E_BAD_ADDRESS;
 			goto error;
 		}
@@ -542,14 +541,14 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 	   with the same branch parameter and will be match-able downstream
 	*/
 	if (!char_msg_val( msg, md5 )) 	{ /* parses transaction key */
-		LOG(L_ERR, "ERROR: forward_request: char_msg_val failed\n");
+		LM_ERR("char_msg_val failed\n");
 		ret=E_UNSPEC;
 		goto error;
 	}
 	msg->hash_index=hash( msg->callid->body, get_cseq(msg)->number);
 	if (!branch_builder( msg->hash_index, 0, md5, 0 /* 0-th branch */,
 				msg->add_to_branch_s, &msg->add_to_branch_len )) {
-		LOG(L_ERR, "ERROR: forward_request: branch_builder failed\n");
+		LM_ERR("branch_builder failed\n");
 		ret=E_UNSPEC;
 		goto error;
 	}
@@ -561,7 +560,7 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 		if (orig_send_sock==0) /* no forced send_sock => find it **/
 			send_info->send_sock=get_send_socket(msg, &send_info->to, proto);
 		if (send_info->send_sock==0){
-			LOG(L_ERR, "forward_req: ERROR: cannot forward to af %d, proto %d "
+			LM_ERR("cannot forward to af %d, proto %d "
 						"no corresponding listening socket\n",
 						send_info->to.s.sa_family, proto);
 			ret=ser_error=E_NO_SOCKET;
@@ -582,7 +581,7 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 			send_info->proto=proto;
 			buf = build_req_buf_from_sip_req(msg, &len, send_info, 0);
 			if (!buf){
-				LOG(L_ERR, "ERROR: forward_request: building failed\n");
+				LM_ERR("building failed\n");
 				ret=E_OUT_OF_MEM; /* most probable */
 				goto error;
 			}
@@ -596,9 +595,8 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 	
 		if (run_onsend(msg, send_info, buf, len)==0){
 			su2ip_addr(&ip, &send_info->to);
-			LOG(L_INFO, "forward_request: request to %s:%d(%d) dropped"
-					" (onsend_route)\n", ip_addr2a(&ip),
-						su_getport(&send_info->to), send_info->proto);
+			LM_INFO("request to %s:%d(%d) dropped (onsend_route)\n",
+				ip_addr2a(&ip), su_getport(&send_info->to), send_info->proto);
 			ser_error=E_OK; /* no error */
 			ret=E_ADM_PROHIBITED;
 #ifdef USE_DNS_FAILOVER
@@ -611,9 +609,8 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 		if (cfg_get(core, core_cfg, use_dst_blacklist)){
 			if (dst_is_blacklisted(send_info, msg)){
 				su2ip_addr(&ip, &send_info->to);
-				LOG(L_DBG, "DEBUG: blacklisted destination:%s:%d (%d)\n",
-							ip_addr2a(&ip), su_getport(&send_info->to),
-							send_info->proto);
+				LM_DBG("blacklisted destination:%s:%d (%d)\n",
+					ip_addr2a(&ip), su_getport(&send_info->to), send_info->proto);
 				ret=ser_error=E_SEND;
 #ifdef USE_DNS_FAILOVER
 				continue; /* try another ip */
@@ -623,7 +620,18 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 			}
 		}
 #endif
+
+		if(unlikely(_forward_set_send_info==1)) {
+			onsnd_info.to=&send_info->to;
+			onsnd_info.send_sock=send_info->send_sock;
+			onsnd_info.buf=buf;
+			onsnd_info.len=len;
+			onsnd_info.msg=msg;
+			p_onsend=&onsnd_info;
+		}
+
 		if (msg_send(send_info, buf, len)<0){
+			p_onsend=0;
 			ret=ser_error=E_SEND;
 #ifdef USE_DST_BLACKLIST
 			(void)dst_blacklist_add(BLST_ERR_SEND, send_info, msg);
@@ -634,6 +642,7 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 			goto error;
 #endif
 		}else{
+			p_onsend=0;
 			ret=ser_error=E_OK;
 			/* sent requests stats */
 			STATS_TX_REQUEST(  msg->first_line.u.request.method_value );
@@ -646,8 +655,7 @@ int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
 			((err=dns_sip_resolve2su(&dns_srv_h, &send_info->to, dst, port,
 										&proto, dns_flags))==0));
 	if ((err!=0) && (err!=-E_DNS_EOR)){
-		LOG(L_ERR, "ERROR:  resolving %.*s host name in uri"
-							" failed: %s [%d] (dropping packet)\n",
+		LM_ERR("resolving %.*s host name in uri failed: %s [%d] (dropping packet)\n",
 									dst->len, ZSW(dst->s),
 									dns_strerror(err), err);
 		ret=ser_error=E_BAD_ADDRESS;
@@ -702,7 +710,7 @@ int update_sock_struct_from_via( union sockaddr_union* to,
 			DBG("update_sock_struct_from_via: using 'rport'\n");
 			port=str2s(via->rport->value.s, via->rport->value.len, &err);
 			if (err){
-				LOG(L_NOTICE, "ERROR: update_sock_struct_from_via: bad rport value(%.*s)\n",
+				LM_ERR("bad rport value(%.*s)\n",
 						via->rport->value.len, via->rport->value.s);
 				port=0;
 			}
@@ -732,9 +740,7 @@ int update_sock_struct_from_via( union sockaddr_union* to,
 	he=sip_resolvehost(name, &port, &proto);
 	
 	if (he==0){
-		LOG(L_NOTICE,
-				"update_sock_struct_from_via:resolve_host(%.*s) failure\n",
-				name->len, name->s);
+		LM_NOTICE("resolve_host(%.*s) failure\n", name->len, name->s);
 		return -1;
 	}
 		
@@ -752,6 +758,7 @@ static int do_forward_reply(struct sip_msg* msg, int mode)
 	struct dest_info dst;
 	unsigned int new_len;
 	int r;
+	struct ip_addr ip;
 #ifdef USE_TCP
 	char* s;
 	int len;
@@ -763,9 +770,8 @@ static int do_forward_reply(struct sip_msg* msg, int mode)
 		if (check_self(&msg->via1->host,
 					msg->via1->port?msg->via1->port:SIP_PORT,
 					msg->via1->proto)!=1){
-			LOG(L_NOTICE, "ERROR: forward_reply: host in first via!=me :"
-					" %.*s:%d\n", msg->via1->host.len, msg->via1->host.s,
-									msg->via1->port);
+			LM_ERR("host in first via!=me : %.*s:%d\n",
+				msg->via1->host.len, msg->via1->host.s, msg->via1->port);
 			/* send error msg back? */
 			goto error;
 		}
@@ -781,13 +787,13 @@ static int do_forward_reply(struct sip_msg* msg, int mode)
 		|| (msg->via2==0) || (msg->via2->error!=PARSE_OK))
 	{
 		/* no second via => error */
-		LOG(L_DBG, "reply cannot be forwarded - no 2nd via\n");
+		LM_DBG("reply cannot be forwarded - no 2nd via\n");
 		goto error;
 	}
 
 	new_buf = build_res_buf_from_sip_res( msg, &new_len);
 	if (!new_buf){
-		LOG(L_ERR, "ERROR: forward_reply: building failed\n");
+		LM_ERR("building failed\n");
 		goto error;
 	}
 
@@ -821,9 +827,8 @@ static int do_forward_reply(struct sip_msg* msg, int mode)
 			len=msg->via1->i->value.len;
 			DBG("forward_reply: i=%.*s\n",len, ZSW(s));
 			if (reverse_hex2int(s, len, (unsigned int*)&dst.id)<0){
-				LOG(L_ERR, "ERROR: forward_reply: bad via i param \"%.*s\"\n",
-						len, ZSW(s));
-					dst.id=0;
+				LM_ERR("bad via i param \"%.*s\"\n", len, ZSW(s));
+				dst.id=0;
 			}
 		}		
 				
@@ -832,11 +837,31 @@ static int do_forward_reply(struct sip_msg* msg, int mode)
 
 	apply_force_send_socket(&dst, msg);
 
+	/* call onsend_route */
+	if(dst.send_sock == NULL) {
+		dst.send_sock=get_send_socket(msg, &dst.to, dst.proto);
+		if (dst.send_sock==0){
+			LM_ERR("cannot forward reply\n");
+			goto done;
+		}
+	}
+	if (onsend_route_enabled(SIP_REPLY)){
+		if (run_onsend(msg, &dst, new_buf, new_len)==0){
+			su2ip_addr(&ip, &(dst.to));
+			LOG(L_ERR, "forward_reply: reply to %s:%d(%d) dropped"
+					" (onsend_route)\n", ip_addr2a(&ip),
+						su_getport(&(dst.to)), dst.proto);
+			goto error; /* error ? */
+		}
+	}
+
 	if (msg_send(&dst, new_buf, new_len)<0)
 	{
 		STATS_RPL_FWD_DROP();
 		goto error;
 	}
+
+	done:
 #ifdef STATS
 	STATS_TX_RESPONSE(  (msg->first_line.u.reply.statuscode/100) );
 #endif
