@@ -23,7 +23,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 /*
  * History:
@@ -351,8 +351,7 @@ static inline void sockaddr2ip_addr(struct ip_addr* ip, struct sockaddr* sa)
 			memcpy(ip->u.addr, &((struct sockaddr_in6*)sa)->sin6_addr, 16);
 			break;
 	default:
-			LOG(L_CRIT, "sockaddr2ip_addr: BUG: unknown address family %d\n",
-					sa->sa_family);
+			LM_CRIT("unknown address family %d\n", sa->sa_family);
 	}
 }
 
@@ -378,8 +377,7 @@ static inline int su_cmp(const union sockaddr_union* s1,
 			return (s1->sin6.sin6_port==s2->sin6.sin6_port)&&
 					(memcmp(&s1->sin6.sin6_addr, &s2->sin6.sin6_addr, 16)==0);
 		default:
-			LOG(L_CRIT,"su_cmp: BUG: unknown address family %d\n",
-						s1->s.sa_family);
+			LM_CRIT("unknown address family %d\n", s1->s.sa_family);
 			return 0;
 	}
 }
@@ -395,8 +393,7 @@ static inline unsigned short su_getport(const union sockaddr_union* su)
 		case AF_INET6:
 			return ntohs(su->sin6.sin6_port);
 		default:
-			LOG(L_CRIT,"su_get_port: BUG: unknown address family %d\n",
-						su->s.sa_family);
+			LM_CRIT("unknown address family %d\n", su->s.sa_family);
 			return 0;
 	}
 }
@@ -414,8 +411,7 @@ static inline void su_setport(union sockaddr_union* su, unsigned short port)
 			 su->sin6.sin6_port=htons(port);
 			 break;
 		default:
-			LOG(L_CRIT,"su_set_port: BUG: unknown address family %d\n",
-						su->s.sa_family);
+			LM_CRIT("unknown address family %d\n", su->s.sa_family);
 	}
 }
 
@@ -436,8 +432,7 @@ static inline void su2ip_addr(struct ip_addr* ip, union sockaddr_union* su)
 					memcpy(ip->u.addr, &su->sin6.sin6_addr, 16);
 					break;
 	default:
-					LOG(L_CRIT,"su2ip_addr: BUG: unknown address family %d\n",
-							su->s.sa_family);
+					LM_CRIT("unknown address family %d\n", su->s.sa_family);
 	}
 }
 
@@ -470,7 +465,7 @@ static inline int init_su( union sockaddr_union* su,
 		su->sin.sin_port=htons(port);
 		break;
 	default:
-		LOG(L_CRIT, "init_ss: BUG: unknown address family %d\n", ip->af);
+		LM_CRIT("unknown address family %d\n", ip->af);
 		return -1;
 	}
 	return 0;
@@ -505,8 +500,7 @@ static inline int hostent2su( union sockaddr_union* su,
 		su->sin.sin_port=htons(port);
 		break;
 	default:
-		LOG(L_CRIT, "hostent2su: BUG: unknown address family %d\n", 
-				he->h_addrtype);
+		LM_CRIT("unknown address family %d\n", he->h_addrtype);
 		return -1;
 	}
 	return 0;
@@ -665,8 +659,7 @@ static inline int ip_addr2sbuf(struct ip_addr* ip, char* buff, int len)
 			return ip4tosbuf(ip->u.addr, buff, len);
 			break;
 		default:
-			LOG(L_CRIT, "BUG: ip_addr2sbuf: unknown address family %d\n",
-					ip->af);
+			LM_CRIT("unknown address family %d\n", ip->af);
 			return 0;
 	}
 	return 0;
@@ -692,7 +685,27 @@ static inline char* ip_addr2a(struct ip_addr* ip)
 	return buff;
 }
 
+/* full address in text representation, including [] for ipv6 */
+static inline char* ip_addr2strz(struct ip_addr* ip)
+{
 
+	static char buff[IP_ADDR_MAX_STR_SIZE+2];
+	char *p;
+	int len;
+
+	p = buff;
+	if(ip->af==AF_INET6) {
+		*p++ = '[';
+	}
+	len=ip_addr2sbuf(ip, p, sizeof(buff)-3);
+	p += len;
+	if(ip->af==AF_INET6) {
+		*p++ = ']';
+	}
+	*p=0;
+
+	return buff;
+}
 
 #define SU2A_MAX_STR_SIZE  (IP6_MAX_STR_SIZE + 2 /* [] */+\
 								1 /* : */ + USHORT2SBUF_MAX_LEN + 1 /* \0 */)
@@ -807,4 +820,6 @@ inline static void init_dst_from_rcv(struct dest_info* dst,
  */
 int ip_addr_match_net(ip_addr_t *iaddr, ip_addr_t *naddr, int mask);
 
+int si_get_signaling_data(struct socket_info *si, str **addr, str **port);
+
 #endif
