@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 /*!
@@ -29,7 +29,6 @@
 #define _STRCOMMON_H_
 
 #include "../../str.h"
-#include "../../md5.h"
 
 /*
  * add backslashes to special characters
@@ -40,8 +39,6 @@ int escape_common(char *dst, char *src, int src_len);
  */
 int unescape_common(char *dst, char *src, int src_len);
 
-void compute_md5(char *dst, char *src, int src_len);
-
 int escape_user(str *sin, str *sout);
 
 int unescape_user(str *sin, str *sout);
