@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * History:
  * -------
@@ -190,6 +190,7 @@
 #include "pv_core.h" /* register core pvars */
 #include "ppcfg.h"
 #include "sock_ut.h"
+#include "async_task.h"
 
 #ifdef DEBUG_DMALLOC
 #include <dmalloc.h>
@@ -227,8 +228,10 @@ Options:\n\
                   (to use both use `-rR`)\n\
     -K           Turn on \"via:\" host checking when forwarding replies\n\
     -d           Debugging mode (multiple -d increase the level)\n\
-    -D no        1..do not fork (almost) anyway, 2..do not daemonize creator\n\
-                  3..daemonize (default)\n\
+    -D           Control how daemonize is done:\n\
+                  -D..do not fork (almost) anyway;\n\
+                  -DD..do not daemonize creator;\n\
+                  -DDD..daemonize (default)\n\
     -E           Log to stderr\n\
     -e           Log messages printed in terminal colors (requires -E)\n"
 #ifdef USE_TCP
@@ -334,6 +337,7 @@ int own_pgid = 0; /* whether or not we have our own pgid (and it's ok
 					 to use kill(0, sig) */
 
 char* mods_dir = MODS_DIR;  /* search path for dyn. loadable modules */
+int   mods_dir_cmd = 0; /* mods dir path set in command lin e*/
 
 char* cfg_file = 0;
 unsigned int maxbuffer = MAX_RECV_BUFFER_SIZE; /* maximum buffer size we do
@@ -377,6 +381,7 @@ int log_stderr = 0;
 int log_color = 0;
 /* set custom app name for syslog printing */
 char *log_name = 0;
+char *log_prefix_fmt = 0;
 pid_t creator_pid = (pid_t) -1;
 int config_check = 0;
 /* check if reply first via host==us */
@@ -422,6 +427,9 @@ int server_id = 0; /* Configurable unique ID of the server */
 /* set timeval for each received sip message */
 int sr_msg_time = 1;
 
+/* onsend_route is executed for replies*/
+int onsend_route_reply = 0;
+
 /* more config stuff */
 int disable_core_dump=0; /* by default enabled */
 int open_files_limit=-1; /* don't touch it by default */
@@ -655,7 +663,7 @@ static void kill_all_children(int signum)
 			if (pt[r].pid) {
 				kill(pt[r].pid, signum);
 			}
-			else LOG(L_CRIT, "BUG: killing: %s > %d no pid!!!\n",
+			else LM_CRIT("killing: %s > %d no pid!!!\n",
 							pt[r].desc, pt[r].pid);
 		}
 		if (!is_main) lock_release(process_lock);
@@ -681,7 +689,7 @@ static void sig_alarm_kill(int signo)
 static void sig_alarm_abort(int signo)
 {
 	/* LOG is not signal safe, but who cares, we are abort-ing anyway :-) */
-	LOG(L_CRIT, "BUG: shutdown timeout triggered, dying...");
+	LM_CRIT("shutdown timeout triggered, dying...");
 	abort();
 }
 
@@ -691,7 +699,7 @@ static void shutdown_children(int sig, int show_status)
 {
 	kill_all_children(sig);
 	if (set_sig_h(SIGALRM, sig_alarm_kill) == SIG_ERR ) {
-		LOG(L_ERR, "ERROR: shutdown: could not install SIGALARM handler\n");
+		LM_ERR("could not install SIGALARM handler\n");
 		/* continue, the process will die anyway if no
 		 * alarm is installed which is exactly what we want */
 	}
@@ -718,7 +726,7 @@ void handle_sigs(void)
 				/* SIGPIPE might be rarely received on use of
 				   exec module; simply ignore it
 				 */
-				LOG(L_WARN, "WARNING: SIGPIPE received and ignored\n");
+				LM_WARN("SIGPIPE received and ignored\n");
 				break;
 		case SIGINT:
 		case SIGTERM:
@@ -727,7 +735,7 @@ void handle_sigs(void)
 				DBG("INT received, program terminates\n");
 			else
 				DBG("SIGTERM received, program terminates\n");
-			LOG(L_NOTICE, "Thank you for flying " NAME "!!!\n");
+			LM_NOTICE("Thank you for flying " NAME "!!!\n");
 			/* shutdown/kill all the children */
 			shutdown_children(SIGTERM, 1);
 			exit(0);
@@ -767,27 +775,27 @@ void handle_sigs(void)
 		case SIGCHLD:
 			while ((chld=waitpid( -1, &chld_status, WNOHANG ))>0) {
 				if (WIFEXITED(chld_status))
-					LOG(L_ALERT, "child process %ld exited normally,"
+					LM_ALERT("child process %ld exited normally,"
 							" status=%d\n", (long)chld,
 							WEXITSTATUS(chld_status));
 				else if (WIFSIGNALED(chld_status)) {
-					LOG(L_ALERT, "child process %ld exited by a signal"
+					LM_ALERT("child process %ld exited by a signal"
 							" %d\n", (long)chld, WTERMSIG(chld_status));
 #ifdef WCOREDUMP
-					LOG(L_ALERT, "core was %sgenerated\n",
+					LM_ALERT("core was %sgenerated\n",
 							 WCOREDUMP(chld_status) ?  "" : "not " );
 #endif
 				}else if (WIFSTOPPED(chld_status))
-					LOG(L_ALERT, "child process %ld stopped by a"
+					LM_ALERT("child process %ld stopped by a"
 								" signal %d\n", (long)chld,
 								 WSTOPSIG(chld_status));
 			}
 #ifndef STOP_JIRIS_CHANGES
 			if (dont_fork) {
-				LOG(L_INFO, "INFO: dont_fork turned on, living on\n");
+				LM_INFO("dont_fork turned on, living on\n");
 				break;
 			}
-			LOG(L_INFO, "INFO: terminating due to SIGCHLD\n");
+			LM_INFO("terminating due to SIGCHLD\n");
 #endif
 			/* exit */
 			shutdown_children(SIGTERM, 1);
@@ -799,7 +807,7 @@ void handle_sigs(void)
 					DBG("SIGHUP received, ignoring it\n");
 					break;
 		default:
-			LOG(L_CRIT, "WARNING: unhandled signal %d\n", sig_flag);
+			LM_CRIT("unhandled signal %d\n", sig_flag);
 	}
 	sig_flag=0;
 }
@@ -832,13 +840,13 @@ void sig_usr(int signo)
 		switch(signo){
 			case SIGPIPE:
 #ifdef SIG_DEBUG /* signal unsafe stuff follows */
-					LOG(L_INFO, "INFO: signal %d received\n", signo);
+					LM_INFO("signal %d received\n", signo);
 #endif
 				break;
 			case SIGINT:
 			case SIGTERM:
 #ifdef SIG_DEBUG /* signal unsafe stuff follows */
-					LOG(L_INFO, "INFO: signal %d received\n", signo);
+					LM_INFO("signal %d received\n", signo);
 					/* print memory stats for non-main too */
 					#ifdef PKG_MALLOC
 					/* make sure we have current cfg values, but update only
@@ -1165,16 +1173,16 @@ int parse_phostport(char* s, char** host, int* hlen,
 end:
 	return 0;
 error_brackets:
-	LOG(L_ERR, "ERROR: parse_phostport: too many brackets in %s\n", s);
+	LM_ERR("too many brackets in %s\n", s);
 	return -1;
 error_colons:
-	LOG(L_ERR, "ERROR: parse_phostport: too many colons in %s\n", s);
+	LM_ERR("too many colons in %s\n", s);
 	return -1;
 error_proto:
-	LOG(L_ERR, "ERROR: parse_phostport: bad protocol in %s\n", s);
+	LM_ERR("bad protocol in %s\n", s);
 	return -1;
 error_port:
-	LOG(L_ERR, "ERROR: parse_phostport: bad port number in %s\n", s);
+	LM_ERR("bad port number in %s\n", s);
 	return -1;
 }
 
@@ -1224,6 +1232,7 @@ int fix_cfg_file(void)
 	
 	if (cfg_file == NULL) cfg_file = CFG_FILE;
 	if (cfg_file[0] == '/') return 0;
+	if (cfg_file[0] == '-' && strlen(cfg_file)==1) return 0;
 	
 	/* cfg_file contains a relative pathname, get the current
 	 * working directory and add it at the beginning
@@ -1271,7 +1280,7 @@ int main_loop(void)
 		setstats( 0 );
 #endif
 		if (udp_listen==0){
-			LOG(L_ERR, "ERROR: no fork mode requires at least one"
+			LM_ERR("no fork mode requires at least one"
 					" udp listen address, exiting...\n");
 			goto error;
 		}
@@ -1316,8 +1325,7 @@ int main_loop(void)
 		} else
 			sendipv6=bind_address;
 		if (udp_listen->next){
-			LOG(L_WARN, "WARNING: using only the first listen address"
-						" (no fork)\n");
+			LM_WARN("using only the first listen address (no fork)\n");
 		}
 
 		/* delay cfg_shmize to the last moment (it must be called _before_
@@ -1325,7 +1333,7 @@ int main_loop(void)
 		   ignored.
 		*/
 		if (cfg_shmize() < 0) {
-			LOG(L_CRIT, "could not initialize shared configuration\n");
+			LM_CRIT("could not initialize shared configuration\n");
 			goto error;
 		}
 	
@@ -1348,6 +1356,9 @@ int main_loop(void)
 		 */
 		cfg_main_set_local();
 
+		/* init log prefix format */
+		log_prefix_init();
+
 		/* init childs with rank==PROC_INIT before forking any process,
 		 * this is a place for delayed (after mod_init) initializations
 		 * (e.g. shared vars that depend on the total number of processes
@@ -1356,8 +1367,7 @@ int main_loop(void)
 		 * for the "main" process with rank PROC_MAIN (make sure things are
 		 * not initialized twice)*/
 		if (init_child(PROC_INIT) < 0) {
-			LOG(L_ERR, "ERROR: main_dontfork: init_child(PROC_INT) --"
-						" exiting\n");
+			LM_ERR("init_child(PROC_INT) -- exiting\n");
 			cfg_main_reset_local();
 			goto error;
 		}
@@ -1368,7 +1378,7 @@ int main_loop(void)
 		/* we need another process to act as the "slow" timer*/
 				pid = fork_process(PROC_TIMER, "slow timer", 0);
 				if (pid<0){
-					LOG(L_CRIT,  "ERROR: main_loop: Cannot fork\n");
+					LM_CRIT("Cannot fork\n");
 					goto error;
 				}
 				if (pid==0){
@@ -1387,7 +1397,7 @@ int main_loop(void)
 				/* we need another process to act as the "main" timer*/
 				pid = fork_process(PROC_TIMER, "timer", 0);
 				if (pid<0){
-					LOG(L_CRIT,  "ERROR: main_loop: Cannot fork\n");
+					LM_CRIT("Cannot fork\n");
 					goto error;
 				}
 				if (pid==0){
@@ -1411,8 +1421,7 @@ int main_loop(void)
 		/* call it also w/ PROC_MAIN to make sure modules that init things 
 		 * only in PROC_MAIN get a chance to run */
 		if (init_child(PROC_MAIN) < 0) {
-			LOG(L_ERR, "ERROR: main_dontfork: init_child(PROC_MAIN) "
-						"-- exiting\n");
+			LM_ERR("init_child(PROC_MAIN) -- exiting\n");
 			goto error;
 		}
 
@@ -1422,7 +1431,7 @@ int main_loop(void)
 		 */
 
 		if (init_child(PROC_SIPINIT) < 0) {
-			LOG(L_ERR, "main_dontfork: init_child failed\n");
+			LM_ERR("init_child failed\n");
 			goto error;
 		}
 		return udp_rcv_loop();
@@ -1552,7 +1561,7 @@ int main_loop(void)
 		   ignored (cfg_shmize() will copy the default cfgs into shmem).
 		*/
 		if (cfg_shmize() < 0) {
-			LOG(L_CRIT, "could not initialize shared configuration\n");
+			LM_CRIT("could not initialize shared configuration\n");
 			goto error;
 		}
 
@@ -1561,6 +1570,9 @@ int main_loop(void)
 		 */
 		cfg_main_set_local();
 
+		/* init log prefix format */
+		log_prefix_init();
+
 		/* init childs with rank==PROC_INIT before forking any process,
 		 * this is a place for delayed (after mod_init) initializations
 		 * (e.g. shared vars that depend on the total number of processes
@@ -1569,8 +1581,7 @@ int main_loop(void)
 		 * for the "main" process with rank PROC_MAIN (make sure things are
 		 * not initialized twice)*/
 		if (init_child(PROC_INIT) < 0) {
-			LOG(L_ERR, "ERROR: main: error in init_child(PROC_INT) --"
-					" exiting\n");
+			LM_ERR("error in init_child(PROC_INT) -- exiting\n");
 			cfg_main_reset_local();
 			goto error;
 		}
@@ -1606,7 +1617,7 @@ int main_loop(void)
 				child_rank++;
 				pid = fork_process(child_rank, si_desc, 1);
 				if (pid<0){
-					LOG(L_CRIT,  "main_loop: Cannot fork\n");
+					LM_CRIT("Cannot fork\n");
 					goto error;
 				}else if (pid==0){
 					/* child */
@@ -1638,7 +1649,7 @@ int main_loop(void)
 					child_rank++;
 					pid = fork_process(child_rank, si_desc, 1);
 					if (pid<0){
-						LOG(L_CRIT,  "main_loop: Cannot fork\n");
+						LM_CRIT("Cannot fork\n");
 						goto error;
 					}else if (pid==0){
 						/* child */
@@ -1662,7 +1673,7 @@ int main_loop(void)
 		/* fork again for the "slow" timer process*/
 		pid = fork_process(PROC_TIMER, "slow timer", 1);
 		if (pid<0){
-			LOG(L_CRIT, "main_loop: cannot fork \"slow\" timer process\n");
+			LM_CRIT("cannot fork \"slow\" timer process\n");
 			goto error;
 		}else if (pid==0){
 			/* child */
@@ -1678,7 +1689,7 @@ int main_loop(void)
 		/* fork again for the "main" timer process*/
 		pid = fork_process(PROC_TIMER, "timer", 1);
 		if (pid<0){
-			LOG(L_CRIT, "main_loop: cannot fork timer process\n");
+			LM_CRIT("cannot fork timer process\n");
 			goto error;
 		}else if (pid==0){
 			/* child */
@@ -1692,7 +1703,7 @@ int main_loop(void)
 	 * to fork  a tcp capable process, the corresponding tcp. comm. fds in
 	 * pt[] must be set before calling tcp_main_loop()) */
 		if (init_child(PROC_MAIN) < 0) {
-			LOG(L_ERR, "ERROR: main: error in init_child\n");
+			LM_ERR("error in init_child\n");
 			goto error;
 		}
 
@@ -1703,8 +1714,7 @@ int main_loop(void)
 				/* start tcp+tls master proc */
 			pid = fork_process(PROC_TCP_MAIN, "tcp main process", 0);
 			if (pid<0){
-				LOG(L_CRIT, "main_loop: cannot fork tcp main process: %s\n",
-							strerror(errno));
+				LM_CRIT("cannot fork tcp main process: %s\n", strerror(errno));
 				goto error;
 			}else if (pid==0){
 				/* child */
@@ -1898,8 +1908,7 @@ int main(int argc, char** argv)
 	
 	/*init pkg mallocs (before parsing cfg or the rest of the cmd line !)*/
 	if (pkg_mem_size)
-		LOG(L_INFO, " private (per process) memory: %ld bytes\n",
-								pkg_mem_size );
+		LM_INFO("private (per process) memory: %ld bytes\n", pkg_mem_size );
 	if (init_pkg_mallocs()==-1)
 		goto error;
 
@@ -1928,6 +1937,7 @@ int main(int argc, char** argv)
 					break;
 			case 'L':
 					mods_dir = optarg;
+					mods_dir_cmd = 1;
 					break;
 			case 'm':
 					shm_mem_size=strtol(optarg, &tmp, 10) * 1024 * 1024;
@@ -1936,8 +1946,7 @@ int main(int argc, char** argv)
 										optarg);
 						goto error;
 					};
-					LOG(L_INFO, "ser: shared memory: %ld bytes\n",
-									shm_mem_size );
+					LM_INFO("shared memory: %ld bytes\n", shm_mem_size );
 					break;
 			case 'M':
 					/* ignore it, it was parsed immediately after startup,
@@ -2056,7 +2065,11 @@ int main(int argc, char** argv)
 	if (fix_cfg_file() < 0) goto error;
 
 	/* load config file or die */
-	cfg_stream=fopen (cfg_file, "r");
+	if (cfg_file[0] == '-' && strlen(cfg_file)==1) {
+		cfg_stream=stdin;
+	} else {
+		cfg_stream=fopen (cfg_file, "r");
+	}
 	if (cfg_stream==0){
 		fprintf(stderr, "ERROR: loading config file(%s): %s\n", cfg_file,
 				strerror(errno));
@@ -2070,13 +2083,12 @@ int main(int argc, char** argv)
 try_again:
 		if (read(rfd, (void*)&seed, sizeof(seed))==-1){
 			if (errno==EINTR) goto try_again; /* interrupted by signal */
-			LOG(L_WARN, "WARNING: could not read from /dev/urandom (%d)\n",
-						errno);
+			LM_WARN("could not read from /dev/urandom (%d)\n", errno);
 		}
 		DBG("read %u from /dev/urandom\n", seed);
 			close(rfd);
 	}else{
-		LOG(L_WARN, "WARNING: could not open /dev/urandom (%d)\n", errno);
+		LM_WARN("could not open /dev/urandom (%d)\n", errno);
 	}
 	seed+=getpid()+time(0);
 	DBG("seeding PRNG with %u\n", seed);
@@ -2095,6 +2107,8 @@ try_again:
 	debug_save = default_core_cfg.debug;
 	if ((yyparse()!=0)||(cfg_errors)){
 		fprintf(stderr, "ERROR: bad config file (%d errors)\n", cfg_errors);
+		if (debug_flag) default_core_cfg.debug = debug_save;
+		pp_ifdef_level_check();
 
 		goto error;
 	}
@@ -2102,6 +2116,7 @@ try_again:
 		fprintf(stderr, "%d config warnings\n", cfg_warnings);
 	}
 	if (debug_flag) default_core_cfg.debug = debug_save;
+	pp_ifdef_level_check();
 	print_rls();
 
 	/* options with higher priority than cfg file */
@@ -2389,53 +2404,53 @@ try_again:
 	if (init_atomic_ops()==-1)
 		goto error;
 	if (init_basex() != 0){
-		LOG(L_CRIT, "could not initialize base* framework\n");
+		LM_CRIT("could not initialize base* framework\n");
 		goto error;
 	}
 	if (sr_cfg_init() < 0) {
-		LOG(L_CRIT, "could not initialize configuration framework\n");
+		LM_CRIT("could not initialize configuration framework\n");
 		goto error;
 	}
 	/* declare the core cfg before the module configs */
 	if (cfg_declare("core", core_cfg_def, &default_core_cfg, cfg_sizeof(core),
 			&core_cfg)
 	) {
-		LOG(L_CRIT, "could not declare the core configuration\n");
+		LM_CRIT("could not declare the core configuration\n");
 		goto error;
 	}
 #ifdef USE_TCP
 	if (tcp_register_cfg()){
-		LOG(L_CRIT, "could not register the tcp configuration\n");
+		LM_CRIT("could not register the tcp configuration\n");
 		goto error;
 	}
 #endif /* USE_TCP */
 	/*init timer, before parsing the cfg!*/
 	if (init_timer()<0){
-		LOG(L_CRIT, "could not initialize timer, exiting...\n");
+		LM_CRIT("could not initialize timer, exiting...\n");
 		goto error;
 	}
 #ifdef USE_DNS_CACHE
 	if (init_dns_cache()<0){
-		LOG(L_CRIT, "could not initialize the dns cache, exiting...\n");
+		LM_CRIT("could not initialize the dns cache, exiting...\n");
 		goto error;
 	}
 #ifdef USE_DNS_CACHE_STATS
 	/* preinitializing before the nubmer of processes is determined */
 	if (init_dns_cache_stats(1)<0){
-		LOG(L_CRIT, "could not initialize the dns cache measurement\n");
+		LM_CRIT("could not initialize the dns cache measurement\n");
 		goto error;
 	}
 #endif /* USE_DNS_CACHE_STATS */
 #endif
 #ifdef USE_DST_BLACKLIST
 	if (init_dst_blacklist()<0){
-		LOG(L_CRIT, "could not initialize the dst blacklist, exiting...\n");
+		LM_CRIT("could not initialize the dst blacklist, exiting...\n");
 		goto error;
 	}
 #ifdef USE_DST_BLACKLIST_STATS
 	/* preinitializing before the number of processes is determined */
 	if (init_dst_blacklist_stats(1)<0){
-		LOG(L_CRIT, "could not initialize the dst blacklist measurement\n");
+		LM_CRIT("could not initialize the dst blacklist measurement\n");
 		goto error;
 	}
 #endif /* USE_DST_BLACKLIST_STATS */
@@ -2447,7 +2462,7 @@ try_again:
 	if (!tcp_disable){
 		/*init tcp*/
 		if (init_tcp()<0){
-			LOG(L_CRIT, "could not initialize tcp, exiting...\n");
+			LM_CRIT("could not initialize tcp, exiting...\n");
 			goto error;
 		}
 	}
@@ -2455,7 +2470,7 @@ try_again:
 #ifdef USE_SCTP
 	if (!sctp_disable){
 		if (sctp_core_init()<0){
-			LOG(L_CRIT, "Could not initialize sctp, exiting...\n");
+			LM_CRIT("Could not initialize sctp, exiting...\n");
 			goto error;
 		}
 	}
@@ -2496,14 +2511,14 @@ try_again:
 #ifdef USE_TLS
 	if (!tls_disable){
 		if (!tls_loaded()){
-			LOG(L_WARN, "WARNING: tls support enabled, but no tls engine "
+			LM_WARN("tls support enabled, but no tls engine "
 						" available (forgot to load the tls module?)\n");
-			LOG(L_WARN, "WARNING: disabling tls...\n");
+			LM_WARN("disabling tls...\n");
 			tls_disable=1;
 		}
 		/* init tls*/
 		if (init_tls()<0){
-			LOG(L_CRIT, "could not initialize tls, exiting...\n");
+			LM_CRIT("could not initialize tls, exiting...\n");
 			goto error;
 		}
 	}
@@ -2519,13 +2534,13 @@ try_again:
 
 #if defined USE_DNS_CACHE && defined USE_DNS_CACHE_STATS
 	if (init_dns_cache_stats(get_max_procs())<0){
-		LOG(L_CRIT, "could not initialize the dns cache measurement\n");
+		LM_CRIT("could not initialize the dns cache measurement\n");
 		goto error;
 	}
 #endif
 #if defined USE_DST_BLACKLIST && defined USE_DST_BLACKLIST_STATS
 	if (init_dst_blacklist_stats(get_max_procs())<0){
-		LOG(L_CRIT, "could not initialize the dst blacklist measurement\n");
+		LM_CRIT("could not initialize the dst blacklist measurement\n");
 		goto error;
 	}
 #endif
