@@ -356,7 +356,7 @@ void sfm_split_frag(struct sfm_block* qm, struct sfm_frag* frag,
 
 
 /* init malloc and return a sfm_block*/
-struct sfm_block* sfm_malloc_init(char* address, unsigned long size)
+struct sfm_block* sfm_malloc_init(char* address, unsigned long size, int type)
 {
 	char* start;
 	char* end;
@@ -392,6 +392,7 @@ struct sfm_block* sfm_malloc_init(char* address, unsigned long size)
 	qm=(struct sfm_block*)start;
 	memset(qm, 0, sizeof(struct sfm_block));
 	qm->size=size;
+	qm->type = type;
 	size-=init_overhead;
 	
 	qm->first_frag=(struct sfm_frag*)(start+ROUNDUP(sizeof(struct sfm_block)));
