@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * History:
  * --------
@@ -48,6 +48,7 @@ typedef struct acc_param {
 	int code;
 	str code_s;
 	str reason;
+	pv_elem_p  elem;
 } acc_param_t;
 
 /* various acc variables */
