@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -35,6 +35,8 @@
 
 #include <jni.h>
 
+#define	APP_NAME	"app_java"
+
 JavaVM *jvm;
 JNIEnv *env;
 jclass KamailioClass;
