@@ -22,7 +22,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 /* History:
  * --------
@@ -36,4 +36,6 @@
 int process_rpc_req(unsigned char* buf, int size, int* bytes_needed,
 					void* sh, void** saved_state);
 
+void binrpc_callbacks_init(void);
+
 #endif
