@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * History:
  * --------
@@ -209,7 +209,7 @@ db1_con_t* bdb_init(const str* _sqlurl)
 	memset(_res, 0, sizeof(db1_con_t) + sizeof(bdb_con_t));
 	_res->tail = (unsigned long)((char*)_res+sizeof(db1_con_t));
 
-	LM_INFO("using database at: %.*s", _s.len, _s.s);
+	LM_INFO("using database at: %.*s\n", _s.len, _s.s);
 	BDB_CON_CONNECTION(_res) = km_bdblib_get_db(&_s);
 	if (!BDB_CON_CONNECTION(_res))
 	{
