@@ -16,10 +16,12 @@
 #include "dlg_hash.h"
 #include "dlg_profile.h"
 #include "dlg_handlers.h"
+#include "dlg_db_handler.h"
 
 #define MAX_LDG_LOCKS  2048
 #define MIN_LDG_LOCKS  2
 
+extern int dlg_db_mode;
 
 /*! global dialog table */
 struct dlg_table *d_table = 0;
@@ -109,7 +111,7 @@ int init_dlg_table(unsigned int size) {
 
     for (i = 0; i < size; i++) {
         memset(&(d_table->entries[i]), 0, sizeof (struct dlg_entry));
-        d_table->entries[i].next_id = rand();
+        d_table->entries[i].next_id = rand() % (3*size);
         d_table->entries[i].lock_idx = i % d_table->locks_no;
     }
 
@@ -189,6 +191,9 @@ inline void destroy_dlg(struct dlg_cell *dlg) {
 
     }
 
+    if (dlg_db_mode)
+    	remove_dialog_in_from_db(dlg);
+
     LM_DBG("About to run dlg callback for destroy\n");
     run_dlg_callbacks(DLGCB_DESTROY, dlg, NULL, NULL, DLG_DIR_NONE, 0);
     LM_DBG("DONE: About to run dlg callback for destroy\n");
@@ -962,6 +967,7 @@ void link_dlg_out(struct dlg_cell *dlg, struct dlg_cell_out *dlg_out, int n) {
 void link_dlg(struct dlg_cell *dlg, int n) {
     struct dlg_entry *d_entry;
 
+    LM_DBG("Linking new dialog with h_entry: %u", dlg->h_entry);
     d_entry = &(d_table->entries[dlg->h_entry]);
 
     dlg_lock(d_table, d_entry);
