@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * History:
  * --------
@@ -43,11 +43,13 @@
 #include "dlg_timer.h"
 #include "dlg_hash.h"
 #include "dlg_req_within.h"
+#include "dlg_db_handler.h"
 
 #define MAX_FWD_HDR        "Max-Forwards: " MAX_FWD CRLF
 #define MAX_FWD_HDR_LEN    (sizeof(MAX_FWD_HDR) - 1)
 
 extern str dlg_extra_hdrs;
+extern int dlg_db_mode;
 
 int free_tm_dlg(dlg_t *td) {
     if (td) {
@@ -210,6 +212,8 @@ void bye_reply_cb(struct cell* t, int type, struct tmcb_params* ps) {
 
     if (new_state == DLG_STATE_DELETED && old_state == DLG_STATE_DELETED) {
         /* trash the dialog from DB and memory */
+        if (dlg_db_mode)
+        	remove_dialog_in_from_db(dlg);
 
         /* force delete from mem */
         unref_dlg(dlg, 1);
