@@ -19,11 +19,12 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
 #include "../../ut.h"
+#include "../../resolve.h"
 #include "dmqnode.h"
 #include "dmq.h"
 
@@ -114,12 +115,12 @@ int set_dmq_node_params(dmq_node_t* node, param_t* params)
 	}
 	status = get_param_value(params, &dmq_node_status_str);
 	if(status) {
-		if(str_strcmp(status, &dmq_node_active_str)) {
-			node->status = DMQ_NODE_ACTIVE;
-		} else if(str_strcmp(status, &dmq_node_timeout_str)) {
-			node->status = DMQ_NODE_ACTIVE;
-		} else if(str_strcmp(status, &dmq_node_disabled_str)) {
+		if(STR_EQ(*status, dmq_node_active_str)) {
 			node->status = DMQ_NODE_ACTIVE;
+		} else if(STR_EQ(*status, dmq_node_timeout_str)) {
+			node->status = DMQ_NODE_TIMEOUT;
+		} else if(STR_EQ(*status, dmq_node_disabled_str)) {
+			node->status = DMQ_NODE_DISABLED;
 		} else {
 			LM_ERR("invalid status parameter: %.*s\n", STR_FMT(status));
 			goto error;
@@ -146,7 +147,11 @@ dmq_node_t* build_dmq_node(str* uri, int shm) {
 	dmq_node_t* ret = NULL;
 	param_hooks_t hooks;
 	param_t* params;
-	
+
+        /* For DNS-Lookups */
+        static char hn[256];
+        struct hostent* he;
+
 	LM_DBG("build_dmq_node %.*s with %s memory\n", STR_FMT(uri), shm?"shm":"private");
 	
 	if(shm) {
@@ -171,7 +176,7 @@ dmq_node_t* build_dmq_node(str* uri, int shm) {
 		}
 	}
 	set_default_dmq_node_params(ret);
-	if(parse_uri(ret->orig_uri.s, ret->orig_uri.len, &ret->uri) < 0) {
+	if(parse_uri(ret->orig_uri.s, ret->orig_uri.len, &ret->uri) < 0 || ret->uri.host.len > 254) {
 		LM_ERR("error parsing uri\n");
 		goto error;
 	}
@@ -199,6 +204,16 @@ dmq_node_t* build_dmq_node(str* uri, int shm) {
 	} else {
 		LM_DBG("no dmqnode params found\n");		
 	}
+	/* resolve hostname */
+	strncpy(hn, ret->uri.host.s, ret->uri.host.len);
+	hn[ret->uri.host.len]='\0';
+	he=resolvehost(hn);
+	if (he==0) {
+		LM_ERR("could not resolve %.*s\n", ret->uri.host.len, ret->uri.host.s);
+		goto error;
+	}
+	hostent2ip_addr(&ret->ip_address, he, 0);
+
 	return ret;
 
 error:
@@ -259,6 +274,15 @@ dmq_node_t* find_dmq_node(dmq_node_list_t* list, dmq_node_t* node)
 dmq_node_t* shm_dup_node(dmq_node_t* node)
 {
 	dmq_node_t* newnode;
+	if (!node) {
+		LM_ERR("node is null\n");
+		return NULL;
+	}
+	if (!node->orig_uri.s) {
+		LM_ERR("nod->orig_uri.s is null\n");
+		return NULL;
+	}
+
 	newnode = shm_malloc(sizeof(dmq_node_t));
 	if(newnode==NULL) {
 		LM_ERR("no more shm\n");
