@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <stddef.h>
@@ -31,12 +31,16 @@
 #include "../../hashes.h"
 #include "../../ut.h"
 #include "../../re.h"
+#include "../../lib/kcore/faked_msg.h"
+#include "../../action.h"
+#include "../../route.h"
 
 #include "ht_api.h"
 #include "ht_db.h"
 
 
 ht_t *_ht_root = NULL;
+ht_cell_t *ht_expired_cell;
 
 typedef struct _keyvalue {
 	str key;
@@ -271,11 +275,34 @@ int ht_init_tables(void)
 {
 	ht_t *ht;
 	int i;
+	char route_name[64];
 
 	ht = _ht_root;
 
 	while(ht)
 	{
+		LM_DBG("initializing htable [%.*s] with nr. of slots: %d\n",
+				ht->name.len, ht->name.s, ht->htsize);
+		if(ht->name.len + sizeof("htable:expired:") < 64)
+		{
+			strcpy(route_name, "htable:expired:");
+			strncat(route_name, ht->name.s, ht->name.len);
+			ht->evrt_expired = route_get(&event_rt, route_name);
+
+			if (ht->evrt_expired < 0
+					|| event_rt.rlist[ht->evrt_expired] == NULL)
+			{
+				ht->evrt_expired = -1;
+				LM_DBG("event route for expired items in [%.*s] does not exist\n",
+						ht->name.len, ht->name.s);
+			} else {
+				LM_DBG("event route for expired items in [%.*s] exists\n",
+						ht->name.len, ht->name.s);
+			}
+		} else {
+			LM_WARN("event route name for expired items in htable [%.*s]"
+					" is too long\n", ht->name.len, ht->name.s);
+		}
 		ht->entries = (ht_entry_t*)shm_malloc(ht->htsize*sizeof(ht_entry_t));
 		if(ht->entries==NULL)
 		{
@@ -563,6 +590,8 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 			/* found */
 			if(now>0 && it->expire!=0 && it->expire<now) {
 				/* entry has expired */
+				ht_handle_expired_record(ht, it);
+
 				if(ht->flags==PV_VAL_INT) {
 					/* initval is integer, use it to create a fresh entry */
 					it->flags &= ~AVP_VAL_STR;
@@ -686,6 +715,8 @@ ht_cell_t* ht_cell_pkg_copy(ht_t *ht, str *name, ht_cell_t *old)
 			/* found */
 			if(ht->htexpire>0 && it->expire!=0 && it->expire<time(NULL)) {
 				/* entry has expired, delete it and return NULL */
+				ht_handle_expired_record(ht, it);
+
 				if(it->prev==NULL)
 					ht->entries[idx].first = it->next;
 				else
@@ -922,6 +953,7 @@ void ht_timer(unsigned int ticks, void *param)
 					if(it->expire!=0 && it->expire<now)
 					{
 						/* expired */
+						ht_handle_expired_record(ht, it);
 						if(it->prev==NULL)
 							ht->entries[i].first = it->next;
 						else
@@ -941,6 +973,46 @@ void ht_timer(unsigned int ticks, void *param)
 	return;
 }
 
+void ht_handle_expired_record(ht_t *ht, ht_cell_t *cell)
+{
+	if(ht->evrt_expired<0)
+		return;
+	ht_expired_cell = cell;
+
+	LM_DBG("running event_route[htable:expired:%.*s]\n",
+			ht->name.len, ht->name.s);
+	ht_expired_run_event_route(ht->evrt_expired);
+
+	ht_expired_cell = NULL;
+}
+
+void ht_expired_run_event_route(int routeid)
+{
+	int backup_rt;
+	sip_msg_t *fmsg;
+
+	if (routeid < 0 || event_rt.rlist[routeid] == NULL)
+	{
+		LM_DBG("route does not exist\n");
+		return;
+	}
+
+	if (faked_msg_init() < 0)
+	{
+		LM_ERR("faked_msg_init() failed\n");
+		return;
+	}
+	fmsg = faked_msg_next();
+	fmsg->parsed_orig_ruri_ok = 0;
+
+	backup_rt = get_route_type();
+
+	set_route_type(EVENT_ROUTE);
+	run_top_route(event_rt.rlist[routeid], fmsg, 0);
+
+	set_route_type(backup_rt);
+}
+
 int ht_set_cell_expire(ht_t *ht, str *name, int type, int_str *val)
 {
 	unsigned int idx;
@@ -1083,6 +1155,38 @@ int ht_rm_cell_re(str *sre, ht_t *ht, int mode)
 	return 0;
 }
 
+int ht_reset_content(ht_t *ht)
+{
+	ht_cell_t *it;
+	ht_cell_t *it0;
+	int i;
+
+	if(ht==NULL)
+		return -1;
+
+	for(i=0; i<ht->htsize; i++)
+	{
+		/* free entries */
+		lock_get(&ht->entries[i].lock);
+		it = ht->entries[i].first;
+		while(it)
+		{
+			it0 = it->next;
+			if(it->prev==NULL)
+				ht->entries[i].first = it->next;
+			else
+				it->prev->next = it->next;
+			if(it->next)
+				it->next->prev = it->prev;
+			ht->entries[i].esize--;
+			ht_cell_free(it);
+			it = it0;
+		}
+		lock_release(&ht->entries[i].lock);
+	}
+	return 0;
+}
+
 int ht_count_cells_re(str *sre, ht_t *ht, int mode)
 {
 	ht_cell_t *it;
@@ -1235,3 +1339,182 @@ int ht_count_cells_re(str *sre, ht_t *ht, int mode)
 	return cnt;
 }
 
+#define HT_ITERATOR_SIZE	4
+#define HT_ITERATOR_NAME_SIZE	32
+
+typedef struct ht_iterator {
+	str name;
+	char bname[HT_ITERATOR_NAME_SIZE];
+	ht_t *ht;
+	int slot;
+	ht_cell_t *it;
+} ht_iterator_t;
+
+static ht_iterator_t _ht_iterators[HT_ITERATOR_SIZE];
+
+void ht_iterator_init(void)
+{
+	memset(_ht_iterators, 0, HT_ITERATOR_SIZE*sizeof(ht_iterator_t));
+}
+
+int ht_iterator_start(str *iname, str *hname)
+{
+	int i;
+	int k;
+
+	k = -1;
+	for(i=0; i<HT_ITERATOR_SIZE; i++)
+	{
+		if(_ht_iterators[i].name.len>0)
+		{
+			if(_ht_iterators[i].name.len==iname->len
+					&& strncmp(_ht_iterators[i].name.s, iname->s, iname->len)==0)
+			{
+				k = i;
+				break;
+			}
+		} else {
+			if(k==-1) k = i;
+		}
+	}
+	if(k==-1)
+	{
+		LM_ERR("no iterator available - max number is %d\n", HT_ITERATOR_SIZE);
+		return -1;
+	}
+	if(_ht_iterators[k].name.len>0)
+	{
+		if(_ht_iterators[k].ht!=NULL && _ht_iterators[k].it!=NULL)
+		{
+			if(_ht_iterators[k].slot>=0 && _ht_iterators[k].slot<_ht_iterators[k].ht->htsize)
+			{
+				lock_release(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+			}
+		}
+	} else {
+		if(iname->len>=HT_ITERATOR_NAME_SIZE)
+		{
+			LM_ERR("iterator name is too big [%.*s] (max %d)\n",
+					iname->len, iname->s, HT_ITERATOR_NAME_SIZE);
+			return -1;
+		}
+		strncpy(_ht_iterators[k].bname, iname->s, iname->len);
+		_ht_iterators[k].bname[iname->len] = '\0';
+	}
+	_ht_iterators[k].it = NULL;
+	_ht_iterators[k].slot = 0;
+	_ht_iterators[k].ht = ht_get_table(hname);
+	if(_ht_iterators[k].ht==NULL)
+	{
+		LM_ERR("cannot get hash table [%.*s]\n", hname->len, hname->s);
+		return -1;
+	}
+	return 0;
+}
+
+int ht_iterator_next(str *iname)
+{
+	int i;
+	int k;
+
+	k = -1;
+	for(i=0; i<HT_ITERATOR_SIZE; i++)
+	{
+		if(_ht_iterators[i].name.len>0)
+		{
+			if(_ht_iterators[i].name.len==iname->len
+					&& strncmp(_ht_iterators[i].name.s, iname->s, iname->len)==0)
+			{
+				k = i;
+				break;
+			}
+		} else {
+			if(k==-1) k = i;
+		}
+	}
+	if(k==-1)
+	{
+		LM_ERR("iterator not found [%.*s]\n", iname->len, iname->s);
+		return -1;
+	}
+	if(_ht_iterators[k].ht==NULL)
+	{
+		LM_ERR("iterator not initialized [%.*s]\n", iname->len, iname->s);
+		return -1;
+	}
+	if(_ht_iterators[k].it==NULL)
+	{
+		/* first execution - start from first slot */
+		_ht_iterators[k].slot=0;
+	} else {
+		_ht_iterators[k].it = _ht_iterators[k].it->next;
+		if(_ht_iterators[k].it!=NULL)
+		{
+			/* next item is in the same slot */
+			return 0;
+		}
+		/* next is not in the same slot - release and try next one */
+		_ht_iterators[k].it = NULL;
+		lock_release(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+		_ht_iterators[k].slot++;
+	}
+
+	for( ; _ht_iterators[k].slot<_ht_iterators[k].ht->htsize; _ht_iterators[k].slot++)
+	{
+		lock_get(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+		if(_ht_iterators[k].ht->entries[_ht_iterators[k].slot].first!=NULL)
+		{
+			_ht_iterators[k].it = _ht_iterators[k].ht->entries[_ht_iterators[k].slot].first;
+			return 0;
+		}
+		lock_release(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+	}
+	return -1;
+}
+
+int ht_iterator_end(str *iname)
+{
+	int i;
+
+	for(i=0; i<HT_ITERATOR_SIZE; i++)
+	{
+		if(_ht_iterators[i].name.len>0)
+		{
+			if(_ht_iterators[i].name.len==iname->len
+					&& strncmp(_ht_iterators[i].name.s, iname->s, iname->len)==0)
+			{
+				if(_ht_iterators[i].ht!=NULL && _ht_iterators[i].it!=NULL)
+				{
+					if(_ht_iterators[i].slot>=0 && _ht_iterators[i].slot<_ht_iterators[i].ht->htsize)
+					{
+						lock_release(&_ht_iterators[i].ht->entries[_ht_iterators[i].slot].lock);
+					}
+				}
+				memset(&_ht_iterators[i], 0, sizeof(ht_iterator_t));
+				return 0;
+			}
+		}
+	}
+
+	return -1;
+}
+
+ht_cell_t* ht_iterator_get_current(str *iname)
+{
+	int i;
+	if(iname==NULL || iname->len<=0)
+		return NULL;
+
+	for(i=0; i<HT_ITERATOR_SIZE; i++)
+	{
+		if(_ht_iterators[i].name.len>0)
+		{
+			if(_ht_iterators[i].name.len==iname->len
+					&& strncmp(_ht_iterators[i].name.s, iname->s, iname->len)==0)
+			{
+				return _ht_iterators[i].it;
+			}
+		}
+	}
+	return NULL;
+}
