@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 		       
 #ifndef _HT_API_H_
@@ -63,6 +63,7 @@ typedef struct _ht
 	int updateexpire;
 	unsigned int htsize;
 	int dmqreplicate;
+	int evrt_expired;
 	ht_entry_t *entries;
 	struct _ht *next;
 } ht_t;
@@ -94,11 +95,20 @@ int ht_db_sync_tables(void);
 
 int ht_has_autoexpire(void);
 void ht_timer(unsigned int ticks, void *param);
+void ht_handle_expired_record(ht_t *ht, ht_cell_t *cell);
+void ht_expired_run_event_route(int routeid);
 int ht_set_cell_expire(ht_t *ht, str *name, int type, int_str *val);
 int ht_get_cell_expire(ht_t *ht, str *name, unsigned int *val);
 
 int ht_rm_cell_re(str *sre, ht_t *ht, int mode);
 int ht_count_cells_re(str *sre, ht_t *ht, int mode);
 ht_t *ht_get_root(void);
+int ht_reset_content(ht_t *ht);
+
+void ht_iterator_init(void);
+int ht_iterator_start(str *iname, str *hname);
+int ht_iterator_next(str *iname);
+int ht_iterator_end(str *iname);
+ht_cell_t* ht_iterator_get_current(str *iname);
 
 #endif
