@@ -39,7 +39,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  */
 
@@ -60,25 +60,40 @@
 #include "../../mod_fix.h"
 
 /* Bindings to PUA */
-//#include "../../modules_k/pua/pua_bind.h"
+#include "../pua/pua_bind.h"
+#include "notify.h"
 
 #include "reg_mod.h"
 #include "save.h"
 #include "service_routes.h"
+#include "lookup.h"
 
 MODULE_VERSION
 
 usrloc_api_t ul;						/**!< Structure containing pointers to usrloc functions*/
 sl_api_t slb;							/**!< SL API structure */
 struct tm_binds tmb;					/**!< TM API structure */
-//pua_api_t pua; 							/**!< PUA API structure */
+pua_api_t pua; 							/**!< PUA API structure */
 
+int publish_reginfo = 0;
+int subscribe_to_reginfo = 0;
+int subscription_expires = 3600;
+int ignore_reg_state = 0;
+int ignore_contact_rxport_check = 0;                             /**!< ignore port checks between received port on message and 
+registration received port. 
+                                                                 * this is useful for example if you register with UDP but possibly send invite over TCP (message too big)*/
 
 time_t time_now;
-char * pcscf_uri = "sip:pcscf.ims.smilecoms.com:4060";
-str pcscf_uri_str;
+
+str pcscf_uri = str_init("sip:pcscf.ims.smilecoms.com:4060");
+str force_icscf_uri = str_init("");
+
 unsigned int pending_reg_expires = 30;			/**!< parameter for expiry time of a pending registration before receiving confirmation from SCSCF */
 
+int is_registered_fallback2ip = 0;
+
+
+
 char* rcv_avp_param = 0;
 unsigned short rcv_avp_type = 0;
 int_str rcv_avp_name;
@@ -95,8 +110,12 @@ static int w_save_pending(struct sip_msg* _m, char* _d, char* _cflags);
 static int w_follows_service_routes(struct sip_msg* _m, char* _d, char* _foo);
 static int w_force_service_routes(struct sip_msg* _m, char* _d, char* _foo);
 static int w_is_registered(struct sip_msg* _m, char* _d, char* _foo);
+static int w_reginfo_handle_notify(struct sip_msg* _m, char* _d, char* _foo);
 
 static int w_assert_identity(struct sip_msg* _m, char* _d, char* _preferred_uri);
+static int w_assert_called_identity(struct sip_msg* _m, char* _d, char* _foo);
+
+static int w_lookup_transport(struct sip_msg* _m, char* _d, char* _uri);
 
 /*! \brief Fixup functions */
 static int domain_fixup(void** param, int param_no);
@@ -118,16 +137,16 @@ inline void pcscf_act_time()
  * Exported functions
  */
 static cmd_export_t cmds[] = {
-	{"pcscf_save",     		(cmd_function)w_save,       	1,  	save_fixup2, 	0,		ONREPLY_ROUTE },
-	{"pcscf_save_pending",	(cmd_function)w_save_pending,       	1,  	save_fixup2, 	0,		REQUEST_ROUTE },
-
-	{"pcscf_follows_service_routes", (cmd_function)w_follows_service_routes,       	1,  	save_fixup2, 	0,		REQUEST_ROUTE },
-	{"pcscf_force_service_routes", (cmd_function)w_force_service_routes,       	1,  	save_fixup2, 	0,		REQUEST_ROUTE },
-
-	{"pcscf_is_registered", (cmd_function)w_is_registered,       	1,  	save_fixup2, 	0,		REQUEST_ROUTE },
-
-	{"pcscf_assert_identity", (cmd_function)w_assert_identity,      2,  	assert_identity_fixup, 	0,		REQUEST_ROUTE },
-
+	{"pcscf_save",     		(cmd_function)w_save,                   1,  	save_fixup2,            0,		ONREPLY_ROUTE },
+	{"pcscf_save_pending",          (cmd_function)w_save_pending,       	1,  	save_fixup2,            0,		REQUEST_ROUTE },
+	{"pcscf_follows_service_routes",(cmd_function)w_follows_service_routes, 1,  	save_fixup2,            0,		REQUEST_ROUTE },
+	{"pcscf_force_service_routes",  (cmd_function)w_force_service_routes,   1,  	save_fixup2,            0,		REQUEST_ROUTE },
+	{"pcscf_is_registered",         (cmd_function)w_is_registered,          1,  	save_fixup2,            0,		REQUEST_ROUTE|ONREPLY_ROUTE },
+	{"pcscf_assert_identity",       (cmd_function)w_assert_identity,        2,  	assert_identity_fixup,  0,		REQUEST_ROUTE },
+	{"pcscf_assert_called_identity",(cmd_function)w_assert_called_identity, 1,  assert_identity_fixup,  0,		ONREPLY_ROUTE },
+	{"reginfo_handle_notify",       (cmd_function)w_reginfo_handle_notify,  1,      domain_fixup,           0,              REQUEST_ROUTE},
+        {"lookup_transport",		(cmd_function)w_lookup_transport,                 1,      domain_fixup,           0,              REQUEST_ROUTE|FAILURE_ROUTE},
+	
 	{0, 0, 0, 0, 0, 0}
 };
 
@@ -136,15 +155,16 @@ static cmd_export_t cmds[] = {
  * Exported parameters
  */
 static param_export_t params[] = {
-	{"pcscf_uri",         	STR_PARAM, &pcscf_uri    						},
-	{"pending_reg_expires",	INT_PARAM, &pending_reg_expires					},
-
-	{"received_avp",       STR_PARAM, &rcv_avp_param       					},
-
-
-//	{"ims_mode",           INT_PARAM, &registrar_ims_mode                  	}, /* 0-PCSCF ; 1-SCSCF */
-//	{"subscription_max_expires",INT_PARAM, &subscription_max_expires       	},
-//	{"subscription_min_expires",INT_PARAM, &subscription_min_expires       	},
+	{"pcscf_uri",                   PARAM_STR, &pcscf_uri                           },
+	{"pending_reg_expires",         INT_PARAM, &pending_reg_expires			},
+	{"received_avp",                PARAM_STR, &rcv_avp_param       		},
+	{"is_registered_fallback2ip",	INT_PARAM, &is_registered_fallback2ip           },
+	{"publish_reginfo",             INT_PARAM, &publish_reginfo                     },
+        {"subscribe_to_reginfo",        INT_PARAM, &subscribe_to_reginfo                },
+        {"subscription_expires",        INT_PARAM, &subscription_expires                },
+        {"ignore_contact_rxport_check", INT_PARAM, &ignore_contact_rxport_check         },
+        {"ignore_reg_state",		INT_PARAM, &ignore_reg_state			},
+	{"force_icscf_uri",		PARAM_STR, &force_icscf_uri			},
 //	{"store_profile_dereg",	INT_PARAM, &store_data_on_dereg},
 	{0, 0, 0}
 };
@@ -184,9 +204,6 @@ int fix_parameters() {
 	str s;
 	pv_spec_t avp_spec;
 
-	pcscf_uri_str.s = pcscf_uri;
-	pcscf_uri_str.len = strlen(pcscf_uri);
-
 	if (rcv_avp_param && *rcv_avp_param) {
 		s.s = rcv_avp_param; s.len = strlen(s.s);
 		if (pv_parse_spec(&s, &avp_spec)==0
@@ -213,7 +230,7 @@ int fix_parameters() {
  */
 static int mod_init(void) {
 	bind_usrloc_t bind_usrloc;
-//	bind_pua_t bind_pua;
+	bind_pua_t bind_pua;
 
 	if (!fix_parameters()) goto error;
 
@@ -242,26 +259,32 @@ static int mod_init(void) {
 	}
 	LM_DBG("Successfully bound to PCSCF Usrloc module\n");
 
-	/* Bind to PUA: */
-//	bind_pua = (bind_pua_t) find_export("bind_pua", 1, 0);
-//	if (!bind_pua) {
-//		LM_ERR("Can't bind pua\n");
-//		return -1;
-//	}
-//	if (bind_pua(&pua) < 0) {
-//		LM_ERR("Can't bind pua\n");
-//		return -1;
-//	}
-//	/* Check for Publish/Subscribe methods */
-//	if (pua.send_publish == NULL) {
-//		LM_ERR("Could not import send_publish\n");
-//		return -1;
-//	}
-//	if (pua.send_subscribe == NULL) {
-//		LM_ERR("Could not import send_subscribe\n");
-//		return -1;
-//	}
-//	LM_DBG("Successfully bound to PUA module\n");
+       if(subscribe_to_reginfo == 1){
+               /* Bind to PUA: */
+               bind_pua = (bind_pua_t) find_export("bind_pua", 1, 0);
+               if (!bind_pua) {
+                       LM_ERR("Can't bind pua\n");
+                       return -1;
+               }
+               if (bind_pua(&pua) < 0) {
+                       LM_ERR("Can't bind pua\n");
+                       return -1;
+               }
+               /* Check for Publish/Subscribe methods */
+               if (pua.send_publish == NULL) {
+                       LM_ERR("Could not import send_publish\n");
+                       return -1;
+               }
+               if (pua.send_subscribe == NULL) {
+                       LM_ERR("Could not import send_subscribe\n");
+                       return -1;
+               }
+	       if (pua.get_subs_list == NULL) {
+                       LM_ERR("Could not import get_subs_list\n");
+                       return -1;
+               }
+	       LM_DBG("Successfully bound to PUA module\n");
+       }
 
 	return 0;
 
@@ -355,6 +378,21 @@ static int w_save(struct sip_msg* _m, char* _d, char* _cflags)
 	return save(_m, (udomain_t*)_d, ((int)(unsigned long)_cflags));
 }
 
+/*! \brief
+ * Wrapper to lookup_transport(location)
+ */
+static int w_lookup_transport(struct sip_msg* _m, char* _d, char* _uri)
+{
+	str uri = {0};
+	if(_uri!=NULL && (fixup_get_svalue(_m, (gparam_p)_uri, &uri)!=0 || uri.len<=0))
+	{
+		LM_ERR("invalid uri parameter\n");
+		return -1;
+	}
+
+	return lookup_transport(_m, (udomain_t*)_d, (uri.len>0)?&uri:NULL);
+}
+
 static int w_save_pending(struct sip_msg* _m, char* _d, char* _cflags)
 {
 	return save_pending(_m, (udomain_t*)_d);
@@ -375,6 +413,11 @@ static int w_is_registered(struct sip_msg* _m, char* _d, char* _foo)
 	return is_registered(_m, (udomain_t*)_d);
 }
 
+static int w_reginfo_handle_notify(struct sip_msg* _m, char* _d, char* _foo)
+{
+       return reginfo_handle_notify(_m, _d, _foo);
+}
+
 static int w_assert_identity(struct sip_msg* _m, char* _d, char* _preferred_uri) {
 	pv_elem_t *model;
 	str identity;
@@ -393,6 +436,10 @@ static int w_assert_identity(struct sip_msg* _m, char* _d, char* _preferred_uri)
 	return assert_identity( _m, (udomain_t*)_d, identity);
 }
 
+static int w_assert_called_identity(struct sip_msg* _m, char* _d, char* _foo) {
+	return assert_called_identity( _m, (udomain_t*)_d);
+}
+
 /*
  * Get the asserted Identity for the current user
  */
