@@ -39,7 +39,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  */
 
@@ -56,7 +56,10 @@
 #include "usrloc.h"
 #include "bin_utils.h"
 #include "subscribe.h"
+#include "usrloc_db.h"
 #include "../../lib/ims/useful_defs.h"
+#include "../../modules/dialog_ng/dlg_load.h"
+#include "../../modules/dialog_ng/dlg_hash.h"
 
 /*! contact matching mode */
 int matching_mode = CONTACT_ONLY;
@@ -66,6 +69,12 @@ int cseq_delay = 20;
 extern int unreg_validity;
 extern int maxcontact_behaviour;
 extern int maxcontact;
+extern int db_mode;
+
+extern int sub_dialog_hash_size;
+extern shtable_t sub_dialog_table;
+
+extern struct dlg_binds dlgb;
 
 /*!
  * \brief Create and initialize new record structure
@@ -109,6 +118,7 @@ int new_impurecord(str* _dom, str* public_identity, int reg_state, int barring,
     if (barring >= 0) { //just in case we call this with no barring -1 will ignore
         (*_r)->barring = barring;
     }
+    (*_r)->send_sar_on_delete = 1; /*defaults to 1 */
     if (ccf1 && ccf1->len > 0) STR_SHM_DUP((*_r)->ccf1, *ccf1, "CCF1");
     if (ccf2 && ccf2->len > 0) STR_SHM_DUP((*_r)->ccf2, *ccf2, "CCF2");
     if (ecf1 && ecf1->len > 0) STR_SHM_DUP((*_r)->ecf1, *ecf1, "ECF1");
@@ -316,6 +326,14 @@ void mem_remove_ucontact(impurecord_t* _r, ucontact_t* _c) {
  * \param _c deleted contact
  */
 void mem_delete_ucontact(impurecord_t* _r, ucontact_t* _c) {
+    
+    struct contact_dialog_data *dialog_data;
+    //tear down dialogs in dialog data list
+    for (dialog_data = _c->first_dialog_data; dialog_data;) {
+        dlgb.lookup_terminate_dlg(dialog_data->h_entry, dialog_data->h_id, NULL );
+        dialog_data = dialog_data->next;
+    }
+    
     mem_remove_ucontact(_r, _c);
     if_update_stat(_r->slot, _r->slot->d->contacts, -1);
     free_ucontact(_c);
@@ -330,8 +348,11 @@ void mem_delete_ucontact(impurecord_t* _r, ucontact_t* _c) {
  */
 static inline void nodb_timer(impurecord_t* _r) {
     ucontact_t* ptr, *t;
+    
+    unsigned int hash_code = 0;
 
     reg_subscriber *s;
+    subs_t* sub_dialog;
 
     get_act_time();
 
@@ -346,6 +367,20 @@ static inline void nodb_timer(impurecord_t* _r) {
             LM_DBG("DBG:registrar_timer: Subscriber with watcher_contact <%.*s> and presentity uri <%.*s> is valid and expires in %d seconds.\n",
                     s->watcher_contact.len, s->watcher_contact.s, s->presentity_uri.len, s->presentity_uri.s,
                     (unsigned int) (s->expires - time(NULL)));
+	    hash_code = core_hash(&s->call_id, &s->to_tag, sub_dialog_hash_size);
+	    LM_DBG("Hash size: <%i>", sub_dialog_hash_size);
+	    LM_DBG("Searching sub dialog hash info with call_id: <%.*s> and ttag <%.*s> ftag <%.*s> and hash code <%i>", s->call_id.len, s->call_id.s, s->to_tag.len, s->to_tag.s, s->from_tag.len, s->from_tag.s, hash_code);
+	    /* search the record in hash table */
+	    lock_get(&sub_dialog_table[hash_code].lock);
+	    sub_dialog= pres_search_shtable(sub_dialog_table, s->call_id, s->to_tag, s->from_tag, hash_code);
+	    if(sub_dialog== NULL)
+	    {
+		LM_ERR("DBG:registrar_timer: Subscription has no dialog record in hash table\n");
+	    }else {
+		LM_DBG("DBG:registrar_timer: Subscription has dialog record in hash table with presentity uri <%.*s>\n", sub_dialog->pres_uri.len, sub_dialog->pres_uri.s);
+	    }
+	    
+	    lock_release(&sub_dialog_table[hash_code].lock);
         }
         s = s->next;
     }
@@ -370,6 +405,10 @@ static inline void nodb_timer(impurecord_t* _r) {
             t = ptr;
             ptr = ptr->next;
 
+			if (db_mode == WRITE_THROUGH && db_delete_ucontact(_r, t) != 0) {
+				LM_ERR("error removing contact from DB [%.*s]... will still remove from memory\n", t->c.len, t->c.s);
+			}
+
             mem_delete_ucontact(_r, t);
             update_stat(_r->slot->d->expires, 1);
         } else {
@@ -433,6 +472,12 @@ int insert_ucontact(impurecord_t* _r, str* _contact, ucontact_info_t* _ci, ucont
         return -1;
     }
 
+    /*DB?*/
+	if (db_mode == WRITE_THROUGH && db_insert_ucontact(_r, *_c) != 0) {
+		LM_ERR("error inserting contact into db");
+		return -1;
+	}
+
     if (exists_ulcb_type(NULL, UL_CONTACT_INSERT)) {
         run_ul_callbacks(NULL, UL_CONTACT_INSERT, _r, *_c);
     }
@@ -451,22 +496,6 @@ int insert_ucontact(impurecord_t* _r, str* _contact, ucontact_info_t* _ci, ucont
  */
 int delete_ucontact(impurecord_t* _r, struct ucontact* _c) {
     int ret = 0;
-    reg_subscriber *s;
-
-    //Richard added this  - fix to remove subscribes that have presentity and watcher uri same as a contact aor that is being removed
-    s = _r->shead;
-    LM_DBG("Checking if there is a subscription to this IMPU that has same watcher contact as this contact");
-    while (s) {
-        
-        LM_DBG("Subscription for this impurecord: watcher uri [%.*s] presentity uri [%.*s] watcher contact [%.*s] ", s->watcher_uri.len, s->watcher_uri.s, 
-                s->presentity_uri.len, s->presentity_uri.s, s->watcher_contact.len, s->watcher_contact.s);
-        LM_DBG("Contact to be removed [%.*s] ", _c->c.len, _c->c.s);
-        if ((s->watcher_contact.len == _c->c.len) && (strncasecmp(s->watcher_contact.s, _c->c.s, _c->c.len) == 0)) {
-            LM_DBG("This contact has a subscription to its own status - so going to delete the subscription");
-            delete_subscriber(_r, s);
-        }
-        s = s->next;
-    }
     
     if (exists_ulcb_type(_c->cbs, UL_CONTACT_DELETE)) {
         run_ul_callbacks(_c->cbs, UL_CONTACT_DELETE, _r, _c);
@@ -475,13 +504,46 @@ int delete_ucontact(impurecord_t* _r, struct ucontact* _c) {
         run_ul_callbacks(_r->cbs, UL_IMPU_DELETE_CONTACT, _r, _c);
     }
 
-    
+	/*DB?*/
+	if (db_mode == WRITE_THROUGH && db_delete_ucontact(_r, _c) != 0) {
+		LM_ERR("error removing contact from DB [%.*s]... will still remove from memory\n", _c->c.len, _c->c.s);
+
+	}
 
     mem_delete_ucontact(_r, _c);
 
     return ret;
 }
 
+
+/* function to convert contact aor to only have data after @ - ie strip user part */
+inline int aor_to_contact(str* aor, str* contact) {
+	char* p;
+	int ret = 0;	//success
+
+	contact->s = aor->s;
+	contact->len = aor->len;
+	if (memcmp(aor->s, "sip:", 4) == 0) {
+		contact->s = aor->s + 4;
+		contact->len-=4;
+	}
+
+	if ((p=memchr(contact->s, '@', contact->len))) {
+		contact->len -= (p - contact->s + 1);
+		contact->s = p+1;
+	}
+
+	if ((p=memchr(contact->s, ';', contact->len))) {
+		contact->len = p - contact->s;
+	}
+
+	if ((p=memchr(contact->s, '>', contact->len))) {
+		contact->len = p - contact->s;
+	}
+
+	return ret;
+}
+
 /*!
  * \brief Match a contact record to a contact string
  * \param ptr contact record
@@ -499,6 +561,27 @@ static inline struct ucontact* contact_match(ucontact_t* ptr, str* _c) {
     return 0;
 }
 
+/*!
+ * \brief Match a contact record to a contact string but only compare the ip port portion
+ * \param ptr contact record
+ * \param _c contact string
+ * \return ptr on successfull match, 0 when they not match
+ */
+static inline struct ucontact* contact_port_ip_match(ucontact_t* ptr, str* _c) {
+    str string_ip_port, contact_ip_port;
+    aor_to_contact(_c, &string_ip_port);//strip userpart from test contact
+
+    while (ptr) {
+	aor_to_contact(&ptr->c, &contact_ip_port);//strip userpart from contact
+	if ((string_ip_port.len == contact_ip_port.len) && !memcmp(string_ip_port.s, contact_ip_port.s, string_ip_port.len)) {
+            return ptr;
+        }
+
+        ptr = ptr->next;
+    }
+    return 0;
+}
+
 /*!
  * \brief Match a contact record to a contact string and callid
  * \param ptr contact record
@@ -576,6 +659,9 @@ int get_ucontact(impurecord_t* _r, str* _c, str* _callid, str* _path, int _cseq,
         case CONTACT_PATH:
             ptr = contact_path_match(_r->contacts, _c, _path);
             break;
+	case CONTACT_PORT_IP_ONLY:
+	    ptr = contact_port_ip_match(_r->contacts, _c);
+	    break;
         default:
             LM_CRIT("unknown matching_mode %d\n", matching_mode);
             return -1;
@@ -680,7 +766,7 @@ void free_ims_subscription_data(ims_subscription *s) {
  * make sure yuo lock the domain before calling this and unlock it afterwards
  * return: 0 on success, -1 on failure
  */
-int update_impurecord(struct udomain* _d, str* public_identity, int reg_state, int barring, int is_primary, ims_subscription** s, str* ccf1, str* ccf2, str* ecf1, str* ecf2, struct impurecord** _r) {
+int update_impurecord(struct udomain* _d, str* public_identity, int reg_state, int send_sar_on_delete, int barring, int is_primary, ims_subscription** s, str* ccf1, str* ccf2, str* ecf1, str* ecf2, struct impurecord** _r) {
     int res;
 
     res = get_impurecord(_d, public_identity, _r);
@@ -704,8 +790,8 @@ int update_impurecord(struct udomain* _d, str* public_identity, int reg_state, i
                 return 0;
             }
         } else {
-            LM_ERR("no IMPU found to update and data not valid to create new one\n");
-            return -1;
+            LM_DBG("no IMPU found to update and data not valid to create new one - not a problem record was probably removed as it has no contacts\n");
+            return 0;
         }
 
     }
@@ -718,6 +804,9 @@ int update_impurecord(struct udomain* _d, str* public_identity, int reg_state, i
         (*_r)->expires = time(NULL) + unreg_validity;
     }
     if (barring >= 0) (*_r)->barring = barring;
+    
+    if (send_sar_on_delete >= 0) (*_r)->send_sar_on_delete = send_sar_on_delete;
+    
     if (ccf1) {
         if ((*_r)->ccf1.s)
             shm_free((*_r)->ccf1.s);
