@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -27,6 +27,8 @@
 
 #include "../../str.h"
 #include "../../parser/msg_parser.h"
+#include "../../lib/kmi/mi.h"
+#include "../../rpc.h"
 
 #define MT_TREE_SVAL	0	
 #define MT_TREE_DW	1
@@ -70,6 +72,8 @@ typedef struct _m_tree
 	unsigned int nrnodes;
 	unsigned int nritems;
 	unsigned int memsize;
+	unsigned int reload_count;
+	unsigned int reload_time;
 	mt_node_t *head;
 	struct _m_tree *next;
 } m_tree_t;
@@ -81,7 +85,7 @@ int mt_add_to_tree(m_tree_t *pt, str *tprefix, str *svalue);
 m_tree_t* mt_get_tree(str *tname);
 m_tree_t* mt_get_first_tree();
 
-is_t* mt_get_tvalue(m_tree_t *pt, str *tomatch);
+is_t* mt_get_tvalue(m_tree_t *pt, str *tomatch, int *len);
 int mt_match_prefix(struct sip_msg *msg, m_tree_t *pt,
 		    str *tomatch, int mode);
 
@@ -103,5 +107,9 @@ int mt_init_list_head(void);
 m_tree_t *mt_add_tree(m_tree_t **dpt, str *tname, str *dbtable,
 		      int type, int multi);
 
+int mt_mi_match_prefix(struct mi_node *rpl, m_tree_t *pt,
+		    str *tomatch, int mode);
+int mt_rpc_match_prefix(rpc_t* rpc, void* ctx, m_tree_t *pt,
+		    str *tomatch, int mode);
 #endif
 
