@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -54,7 +54,6 @@ static int fixup_redis_cmd6(void** param, int param_no);
 
 static int w_redis_free_reply(struct sip_msg* msg, char* res);
 
-static int  mod_init(void);
 static void mod_destroy(void);
 static int  child_init(int rank);
 
@@ -84,7 +83,7 @@ static cmd_export_t cmds[]={
 };
 
 static param_export_t params[]={
-	{"server",         STR_PARAM|USE_FUNC_PARAM, (void*)redis_srv_param},
+	{"server",         PARAM_STRING|USE_FUNC_PARAM, (void*)redis_srv_param},
 	{0, 0, 0}
 };
 
@@ -97,7 +96,7 @@ struct module_exports exports = {
 	0,              /* exported MI functions */
 	mod_pvs,        /* exported pseudo-variables */
 	0,              /* extra processes */
-	mod_init,       /* module initialization function */
+	0,       /* module initialization function */
 	0,              /* response function */
 	mod_destroy,    /* destroy function */
 	child_init      /* per child init function */
@@ -105,15 +104,6 @@ struct module_exports exports = {
 
 
 
-/**
- * init module function
- */
-static int mod_init(void)
-{
-	/* success code */
-	return 0;
-}
-
 /* each child get a new connection to the database */
 static int child_init(int rank)
 {
