@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 
@@ -189,7 +189,7 @@ int clean_puadb( int update_period, int min_expires )
 	if (RES_ROW_N(res) == 0)
 	{
 		/* no match */ 
-		LM_INFO( "No records matched for clean\n");
+		LM_DBG( "No records matched for clean\n");
 		pua_dbf.free_result(pua_db, res);
 		return(0);
 	}
