@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * History:
  * --------
@@ -27,7 +27,7 @@
  */
 
 #ifndef SIP_XMPP_PRES_H
-#define SIP_XMPP__PRES_H
+#define SIP_XMPP_PRES_H
 
 #include "../pua/pua_bind.h"
 
