@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef _PV_CORE_H_
@@ -132,6 +132,9 @@ int pv_get_srcip(struct sip_msg *msg, pv_param_t *param,
 int pv_get_srcport(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *res);
 
+int pv_get_srcaddr_uri(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res);
+
 int pv_get_rcvip(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *res);
 
@@ -320,5 +323,10 @@ int pv_parse__s_name(pv_spec_p sp, str *in);
 int pv_get__s(sip_msg_t *msg, pv_param_t *param,
 		pv_value_t *res);
 
+int pv_parse_expires_name(pv_spec_p sp, str *in);
+
+int pv_get_expires(sip_msg_t *msg, pv_param_t *param,
+		pv_value_t *res);
+
 #endif
 
