@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -47,6 +47,7 @@
 #include "../../parser/parse_nameaddr.h"
 
 #include "../../lib/kcore/strcommon.h"
+#include "../../lib/srutils/shautils.h"
 #include "pv_trans.h"
 
 
@@ -235,6 +236,36 @@ int tr_eval_string(struct sip_msg *msg, tr_param_t *tp, int subtype,
 			val->rs.s = _tr_buffer;
 			val->rs.len = MD5_LEN;
 			break;
+		case TR_S_SHA256:
+			if(!(val->flags&PV_VAL_STR))
+				val->rs.s = int2str(val->ri, &val->rs.len);
+			compute_sha256(_tr_buffer, (u_int8_t*)val->rs.s, val->rs.len);
+			_tr_buffer[SHA256_DIGEST_STRING_LENGTH -1] = '\0';
+			val->flags = PV_VAL_STR;
+			val->ri = 0;
+			val->rs.s = _tr_buffer;
+			val->rs.len = SHA256_DIGEST_STRING_LENGTH -1 ;
+			break;
+		case TR_S_SHA384:
+			if(!(val->flags&PV_VAL_STR))
+				val->rs.s = int2str(val->ri, &val->rs.len);
+			compute_sha384(_tr_buffer, (u_int8_t*)val->rs.s, val->rs.len);
+			_tr_buffer[SHA384_DIGEST_STRING_LENGTH -1] = '\0';
+			val->flags = PV_VAL_STR;
+			val->ri = 0;
+			val->rs.s = _tr_buffer;
+			val->rs.len = SHA384_DIGEST_STRING_LENGTH -1;
+			break;
+		case TR_S_SHA512:
+			if(!(val->flags&PV_VAL_STR))
+				val->rs.s = int2str(val->ri, &val->rs.len);
+			compute_sha512(_tr_buffer, (u_int8_t*)val->rs.s, val->rs.len);
+			_tr_buffer[SHA512_DIGEST_STRING_LENGTH -1] = '\0';
+			val->flags = PV_VAL_STR;
+			val->ri = 0;
+			val->rs.s = _tr_buffer;
+			val->rs.len = SHA512_DIGEST_STRING_LENGTH -1;
+			break;
 		case TR_S_ENCODEHEXA:
 			if(!(val->flags&PV_VAL_STR))
 				val->rs.s = int2str(val->ri, &val->rs.len);
@@ -1908,6 +1939,15 @@ char* tr_parse_string(str* in, trans_t *t)
 	} else if(name.len==3 && strncasecmp(name.s, "md5", 3)==0) {
 		t->subtype = TR_S_MD5;
 		goto done;
+	} else if(name.len==6 && strncasecmp(name.s, "sha256", 6)==0) {
+		t->subtype = TR_S_SHA256;
+		goto done;
+	} else if(name.len==6 && strncasecmp(name.s, "sha384", 6)==0) {
+		t->subtype = TR_S_SHA384;
+		goto done;
+	} else if(name.len==6 && strncasecmp(name.s, "sha512", 6)==0) {
+		t->subtype = TR_S_SHA512;
+		goto done;
 	} else if(name.len==7 && strncasecmp(name.s, "tolower", 7)==0) {
 		t->subtype = TR_S_TOLOWER;
 		goto done;
