@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -122,6 +122,21 @@ int regapi_set_q_override(struct sip_msg *msg, str *new_q)
 	return set_q_override(msg, _q);
 }
 
+/**
+ *
+ */
+int regapi_lookup_to_dset(struct sip_msg *msg, char *table, str *uri)
+{
+	udomain_t* d;
+
+	if(ul.get_udomain(table, &d)<0)
+	{
+		LM_ERR("usrloc domain [%s] not found\n", table);
+		return -1;
+	}
+	return lookup_to_dset(msg, d, uri);
+}
+
 /**
  *
  */
@@ -135,6 +150,7 @@ int bind_registrar(registrar_api_t* api)
 	api->save_uri   = regapi_save_uri;
 	api->lookup     = regapi_lookup;
 	api->lookup_uri = regapi_lookup_uri;
+	api->lookup_to_dset = regapi_lookup_to_dset;
 	api->registered = regapi_registered;
 	api->set_q_override = regapi_set_q_override;
 
