@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -42,6 +42,7 @@ int regapi_lookup(struct sip_msg *msg, char *table);
 
 typedef int (*regapi_lookup_uri_f)(struct sip_msg *msg, char *table, str *uri);
 int regapi_lookup_uri(struct sip_msg *msg, char *table, str *uri);
+int regapi_lookup_to_dset(struct sip_msg *msg, char *table, str *uri);
 
 typedef int (*regapi_set_q_override_f)(struct sip_msg *msg, str *new_q);
 int regapi_set_q_override(struct sip_msg *msg, str *new_q);
@@ -54,6 +55,7 @@ typedef struct registrar_api {
 	regapi_save_uri_f   save_uri;
 	regapi_lookup_f     lookup;
 	regapi_lookup_uri_f lookup_uri;
+	regapi_lookup_uri_f lookup_to_dset;
 	regapi_lookup_f     registered;
 	regapi_set_q_override_f set_q_override;
 } registrar_api_t;
