@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *
  */
@@ -35,6 +35,8 @@ enum {
     SCA_CALL_INFO_SHARED_CALLER = (1 << 0),
     SCA_CALL_INFO_SHARED_CALLEE = (1 << 1),
 };
+#define SCA_CALL_INFO_SHARED_BOTH \
+	(SCA_CALL_INFO_SHARED_CALLER | SCA_CALL_INFO_SHARED_CALLEE)
 
 struct _sca_call_info {
     str		sca_uri;
