@@ -3,7 +3,7 @@
  *
  * hep related structure
  *
- * Copyright (C) 2011 Alexandr Dubovikov (QSC AG) (alexandr.dubovikov@gmail.com)
+ * Copyright (C) 2011-2014 Alexandr Dubovikov (QSC AG) (alexandr.dubovikov@gmail.com)
  *
  * This file is part of Kamailio, a free SIP server.
  *
@@ -19,10 +19,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#ifndef _sipcapture_h
+#define _sipcapture_h
+
 struct _sipcapture_object {
 	str method;
 	str reply_reason;
@@ -64,3 +67,9 @@ struct _sipcapture_object {
 	stat_var *stat;
 #endif
 };
+
+
+struct hep_generic_recv;
+int receive_logging_json_msg(char * buf, unsigned int len, struct hep_generic_recv *hg, char *log_table);
+
+#endif /* _sipcapture_h */
