@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -91,10 +91,11 @@ static int mod_init(void);
 static int siptrace_init_rpc(void);
 static int child_init(int rank);
 static void destroy(void);
-static int sip_trace(struct sip_msg*, char*, char*);
+static int sip_trace(struct sip_msg*, struct dest_info*, char*);
+static int fixup_siptrace(void ** param, int param_no);
 
 static int sip_trace_store_db(struct _siptrace_data* sto);
-static int trace_send_duplicate(char *buf, int len);
+static int trace_send_duplicate(char *buf, int len, struct dest_info*);
 
 static void trace_onreq_in(struct cell* t, int type, struct tmcb_params *ps);
 static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps);
@@ -103,7 +104,7 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps);
 static void trace_sl_onreply_out(sl_cbp_t *slcb);
 static void trace_sl_ack_in(sl_cbp_t *slcb);
 
-static int trace_send_hep_duplicate(str *body, str *from, str *to);
+static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_info*);
 static int pipport2su (char *pipport, union sockaddr_union *tmp_su, unsigned int *proto);
 
 
@@ -140,6 +141,9 @@ int hep_capture_id = 1;
 int xheaders_write = 0;
 int xheaders_read = 0;
 
+str force_send_sock_str = {0, 0};
+struct sip_uri * force_send_sock_uri = 0;
+
 str    dup_uri_str      = {0, 0};
 struct sip_uri *dup_uri = 0;
 
@@ -169,7 +173,7 @@ db_func_t db_funcs;      		/*!< Database functions */
  */
 static cmd_export_t cmds[] = {
 	{"sip_trace", (cmd_function)sip_trace, 0, 0, 0, ANY_ROUTE},
-	{"sip_trace", (cmd_function)sip_trace, 1, 0, 0, ANY_ROUTE},
+    {"sip_trace", (cmd_function)sip_trace, 1, fixup_siptrace, 0, ANY_ROUTE},
 	{0, 0, 0, 0, 0, 0}
 };
 
@@ -178,29 +182,30 @@ static cmd_export_t cmds[] = {
  * Exported parameters
  */
 static param_export_t params[] = {
-	{"db_url",             STR_PARAM, &db_url.s             },
-	{"table",              STR_PARAM, &siptrace_table.s     },
-	{"date_column",        STR_PARAM, &date_column.s        },
-	{"callid_column",      STR_PARAM, &callid_column.s      },
-	{"traced_user_column", STR_PARAM, &traced_user_column.s },
-	{"msg_column",         STR_PARAM, &msg_column.s         },
-	{"method_column",      STR_PARAM, &method_column.s      },
-	{"status_column",      STR_PARAM, &status_column.s      },
-	{"fromip_column",      STR_PARAM, &fromip_column.s      },
-	{"toip_column",        STR_PARAM, &toip_column.s        },
-	{"fromtag_column",     STR_PARAM, &fromtag_column.s     },
-	{"direction_column",   STR_PARAM, &direction_column.s   },
+	{"db_url",             PARAM_STR, &db_url            },
+	{"table",              PARAM_STR, &siptrace_table     },
+	{"date_column",        PARAM_STR, &date_column        },
+	{"callid_column",      PARAM_STR, &callid_column      },
+	{"traced_user_column", PARAM_STR, &traced_user_column },
+	{"msg_column",         PARAM_STR, &msg_column         },
+	{"method_column",      PARAM_STR, &method_column      },
+	{"status_column",      PARAM_STR, &status_column      },
+	{"fromip_column",      PARAM_STR, &fromip_column      },
+	{"toip_column",        PARAM_STR, &toip_column        },
+	{"fromtag_column",     PARAM_STR, &fromtag_column     },
+	{"direction_column",   PARAM_STR, &direction_column   },
 	{"trace_flag",         INT_PARAM, &trace_flag           },
 	{"trace_on",           INT_PARAM, &trace_on             },
-	{"traced_user_avp",    STR_PARAM, &traced_user_avp_str.s},
-	{"trace_table_avp",    STR_PARAM, &trace_table_avp_str.s},
-	{"duplicate_uri",      STR_PARAM, &dup_uri_str.s        },
+	{"traced_user_avp",    PARAM_STR, &traced_user_avp_str},
+	{"trace_table_avp",    PARAM_STR, &trace_table_avp_str},
+	{"duplicate_uri",      PARAM_STR, &dup_uri_str        },
 	{"trace_to_database",  INT_PARAM, &trace_to_database    },
-	{"trace_local_ip",     STR_PARAM, &trace_local_ip.s     },
+	{"trace_local_ip",     PARAM_STR, &trace_local_ip     },
 	{"trace_sl_acks",      INT_PARAM, &trace_sl_acks        },
 	{"xheaders_write",     INT_PARAM, &xheaders_write       },
 	{"xheaders_read",      INT_PARAM, &xheaders_read        },
 	{"hep_mode_on",        INT_PARAM, &hep_mode_on          },	 
+    {"force_send_sock",    PARAM_STR, &force_send_sock_str	},
 	{"hep_version",        INT_PARAM, &hep_version          },
 	{"hep_capture_id",     INT_PARAM, &hep_capture_id       },	        
 	{"trace_delayed",      INT_PARAM, &trace_delayed        },
@@ -274,27 +279,6 @@ static int mod_init(void)
 		return -1;
 	}
 
-	db_url.len = strlen(db_url.s);
-	siptrace_table.len = strlen(siptrace_table.s);
-	date_column.len = strlen(date_column.s);
-	callid_column.len = strlen(callid_column.s);
-	traced_user_column.len = strlen(traced_user_column.s);
-	msg_column.len = strlen(msg_column.s);
-	method_column.len = strlen(method_column.s);
-	status_column.len = strlen(status_column.s);
-	fromip_column.len = strlen(fromip_column.s);
-	toip_column.len = strlen(toip_column.s);
-	fromtag_column.len = strlen(fromtag_column.s);
-	direction_column.len = strlen(direction_column.s);
-	if (traced_user_avp_str.s)
-		traced_user_avp_str.len = strlen(traced_user_avp_str.s);
-	if (trace_table_avp_str.s)
-		trace_table_avp_str.len = strlen(trace_table_avp_str.s);
-	if (dup_uri_str.s)
-		dup_uri_str.len = strlen(dup_uri_str.s);
-	if (trace_local_ip.s)
-		trace_local_ip.len = strlen(trace_local_ip.s);
-
 	if (trace_flag<0 || trace_flag>(int)MAX_FLAG)
 	{
 		LM_ERR("invalid trace flag %d\n", trace_flag);
@@ -310,6 +294,11 @@ static int mod_init(void)
 
 	*trace_to_database_flag = trace_to_database;
 
+	if(hep_version != 1 && hep_version != 2) {
+	    LM_ERR("unsupported version of HEP");
+	    return -1;
+	}
+
 	/* Find a database module if needed */
 	if(trace_to_database_flag!=NULL && *trace_to_database_flag!=0) {
 		if (db_bind_mod(&db_url, &db_funcs))
@@ -384,7 +373,6 @@ static int mod_init(void)
 
 	if(dup_uri_str.s!=0)
 	{
-		dup_uri_str.len = strlen(dup_uri_str.s);
 		dup_uri = (struct sip_uri *)pkg_malloc(sizeof(struct sip_uri));
 		if(dup_uri==0)
 		{
@@ -399,6 +387,23 @@ static int mod_init(void)
 		}
 	}
 
+	if(force_send_sock_str.s!=0)
+	{
+	    force_send_sock_str.len = strlen(force_send_sock_str.s);
+	    force_send_sock_uri = (struct sip_uri *)pkg_malloc(sizeof(struct sip_uri));
+	    if(force_send_sock_uri==0)
+	    {
+	        LM_ERR("no more pkg memory left\n");
+	        return -1;
+	    }
+	    memset(force_send_sock_uri, 0, sizeof(struct sip_uri));
+	    if(parse_uri(force_send_sock_str.s, force_send_sock_str.len, force_send_sock_uri)<0)
+	    {
+	        LM_ERR("bad dup uri\n");
+	        return -1;
+	    }
+	}
+
 	if(traced_user_avp_str.s && traced_user_avp_str.len > 0)
 	{
 		if (pv_parse_spec(&traced_user_avp_str, &avp_spec)==0
@@ -709,7 +714,7 @@ static int sip_trace_xheaders_free(struct _siptrace_data *sto)
 	return 0;
 }
 
-static int sip_trace_store(struct _siptrace_data *sto)
+static int sip_trace_store(struct _siptrace_data *sto, struct dest_info *dst)
 {
 	if(sto==NULL)
 	{
@@ -726,8 +731,8 @@ static int sip_trace_store(struct _siptrace_data *sto)
 	if (sip_trace_xheaders_write(sto) != 0)
 		return -1;
 
-	if(hep_mode_on) trace_send_hep_duplicate(&sto->body, &sto->fromip, &sto->toip);
-	else trace_send_duplicate(sto->body.s, sto->body.len);
+	if(hep_mode_on) trace_send_hep_duplicate(&sto->body, &sto->fromip, &sto->toip, dst);
+    else trace_send_duplicate(sto->body.s, sto->body.len, dst);
 
 	if (sip_trace_xheaders_free(sto) != 0)
 		return -1;
@@ -866,11 +871,76 @@ error:
 	return -1;
 }
 
-static int sip_trace(struct sip_msg *msg, char *dir, char *s2)
+static int fixup_siptrace(void** param, int param_no) {
+	char *duri = (char*) *param;
+	struct sip_uri dup_uri;
+	struct dest_info *dst = NULL;
+	struct proxy_l * p = NULL;
+	str dup_uri_str = { 0, 0 };
+
+	if (param_no != 1) {
+		LM_DBG("params:%s\n", (char*)*param);
+		return 0;
+	}
+	if (!(*duri)) {
+		LM_ERR("invalid dup URI\n");
+		return -1;
+	}
+	LM_DBG("sip_trace URI:%s\n", (char*)*param);
+
+	dup_uri_str.s = duri;
+	dup_uri_str.len = strlen(dup_uri_str.s);
+	memset(&dup_uri, 0, sizeof(struct sip_uri));
+
+	if (parse_uri(dup_uri_str.s, dup_uri_str.len, &dup_uri) < 0) {
+		LM_ERR("bad dup uri\n");
+		return -1;
+	}
+
+	dst = (struct dest_info *) pkg_malloc(sizeof(struct dest_info));
+	if (dst == 0) {
+		LM_ERR("no more pkg memory left\n");
+		return -1;
+	}
+	init_dest_info(dst);
+	/* create a temporary proxy*/
+	dst->proto = PROTO_UDP;
+	p = mk_proxy(&dup_uri.host, (dup_uri.port_no) ? dup_uri.port_no : SIP_PORT,
+			dst->proto);
+	if (p == 0) {
+		LM_ERR("bad host name in uri\n");
+		pkg_free(dst);
+		return -1;
+	}
+	hostent2su(&dst->to, &p->host, p->addr_idx, (p->port) ? p->port : SIP_PORT);
+
+	pkg_free(*param);
+	/* free temporary proxy*/
+	if (p) {
+		free_proxy(p); /* frees only p content, not p itself */
+		pkg_free(p);
+	}
+
+	*param = (void*) dst;
+	return 0;
+}
+
+static int sip_trace(struct sip_msg *msg, struct dest_info * dst, char *dir)
 {
 	struct _siptrace_data sto;
 	struct onsend_info *snd_inf = NULL;
 
+	if (dst){
+	    if (dst->send_sock == 0){
+	        dst->send_sock=get_send_socket(0, &dst->to, dst->proto);
+	        if (dst->send_sock==0){
+	            LM_ERR("can't forward to af %d, proto %d no corresponding"
+	                    " listening socket\n", dst->to.s.sa_family, dst->proto);
+	            return -1;
+	        }
+	    }
+	}
+
 	if(msg==NULL) {
 		LM_DBG("nothing to trace\n");
 		return -1;
@@ -956,7 +1026,7 @@ static int sip_trace(struct sip_msg *msg, char *dir, char *s2)
 		sto.stat = siptrace_req;
 	}
 #endif
-	return sip_trace_store(&sto);
+	return sip_trace_store(&sto, dst);
 }
 
 #define trace_is_off(_msg) \
@@ -1138,7 +1208,7 @@ static void trace_onreq_out(struct cell* t, int type, struct tmcb_params *ps)
 	sto.stat = siptrace_req;
 #endif
 
-	sip_trace_store(&sto);
+	sip_trace_store(&sto, NULL);
 	return;
 }
 
@@ -1209,7 +1279,7 @@ static void trace_onreply_in(struct cell* t, int type, struct tmcb_params *ps)
 	sto.stat = siptrace_rpl;
 #endif
 
-	sip_trace_store(&sto);
+	sip_trace_store(&sto, NULL);
 	return;
 }
 
@@ -1318,7 +1388,7 @@ static void trace_onreply_out(struct cell* t, int type, struct tmcb_params *ps)
 	sto.stat = siptrace_rpl;
 #endif
 
-	sip_trace_store(&sto);
+	sip_trace_store(&sto, NULL);
 	return;
 }
 
@@ -1404,7 +1474,7 @@ static void trace_sl_onreply_out(sl_cbp_t *slcbp)
 	sto.stat = siptrace_rpl;
 #endif
 
-	sip_trace_store(&sto);
+	sip_trace_store(&sto, NULL);
 	return;
 }
 
@@ -1456,10 +1526,10 @@ static struct mi_root* sip_trace_mi(struct mi_root* cmd_tree, void* param )
 	}
 }
 
-static int trace_send_duplicate(char *buf, int len)
+static int trace_send_duplicate(char *buf, int len, struct dest_info *dst2)
 {
 	struct dest_info dst;
-	struct proxy_l * p;
+	struct proxy_l * p = NULL;
 
 	if(buf==NULL || len <= 0)
 		return -1;
@@ -1478,34 +1548,50 @@ static int trace_send_duplicate(char *buf, int len)
 		return -1;
 	}
 
-	hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
+	if (!dst2){
+	    init_dest_info(&dst);
+	    /* create a temporary proxy*/
+	    dst.proto = PROTO_UDP;
+	    p=mk_proxy(&dup_uri->host, (dup_uri->port_no)?dup_uri->port_no:SIP_PORT,
+	             dst.proto);
+	    if (p==0){
+	        LM_ERR("bad host name in uri\n");
+	        return -1;
+	    }
+	    hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
 
-	dst.send_sock=get_send_socket(0, &dst.to, dst.proto);
-	if (dst.send_sock==0)
-	{
-		LM_ERR("can't forward to af %d, proto %d no corresponding"
-				" listening socket\n", dst.to.s.sa_family, dst.proto);
-		goto error;
+	    dst.send_sock=get_send_socket(0, &dst.to, dst.proto);
+	    if (dst.send_sock==0){
+	        LM_ERR("can't forward to af %d, proto %d no corresponding"
+	                " listening socket\n", dst.to.s.sa_family, dst.proto);
+	        goto error;
+	    }
 	}
 
-	if (msg_send(&dst, buf, len)<0)
+	if (msg_send((dst2)?dst2:&dst, buf, len)<0)
 	{
 		LM_ERR("cannot send duplicate message\n");
 		goto error;
 	}
 
-	free_proxy(p); /* frees only p content, not p itself */
-	pkg_free(p);
+	if (p){
+	    free_proxy(p); /* frees only p content, not p itself */
+	    pkg_free(p);
+	}
 	return 0;
 error:
+    if (p){
 	free_proxy(p); /* frees only p content, not p itself */
 	pkg_free(p);
+    }
 	return -1;
 }
 
-static int trace_send_hep_duplicate(str *body, str *from, str *to)
+static int trace_send_hep_duplicate(str *body, str *from, str *to, struct dest_info * dst2)
 {
 	struct dest_info dst;
+	struct socket_info *si;
+	struct dest_info* dst_fin = NULL;
 	struct proxy_l * p=NULL /* make gcc happy */;
 	void* buffer = NULL;
 	union sockaddr_union from_su;
@@ -1550,6 +1636,7 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to)
 		goto error;
 	}
 
+    if (!dst2){
 	init_dest_info(&dst);
 	/* create a temporary proxy*/
 	dst.proto = PROTO_UDP;
@@ -1562,14 +1649,33 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to)
 	}
 
 	hostent2su(&dst.to, &p->host, p->addr_idx, (p->port)?p->port:SIP_PORT);
+	LM_DBG("setting up the socket_info\n");
+	dst_fin = &dst;
+    } else {
+        dst_fin = dst2;
+    }
 
-	dst.send_sock=get_send_socket(0, &dst.to, dst.proto);
-	if (dst.send_sock==0)
-	{
-		LM_ERR("can't forward to af %d, proto %d no corresponding"
-				" listening socket\n", dst.to.s.sa_family, dst.proto);
-		goto error;
-	}
+    if (force_send_sock_str.s) {
+        LM_DBG("force_send_sock activated, grep for the sock_info\n");
+        si = grep_sock_info(&force_send_sock_uri->host,
+                (force_send_sock_uri->port_no)?force_send_sock_uri->port_no:SIP_PORT,
+                PROTO_UDP);
+        if (!si) {
+             LM_WARN("cannot grep socket info\n");
+        } else {
+            LM_DBG("found socket while grep: [%.*s] [%.*s]\n", si->name.len, si->name.s, si->address_str.len, si->address_str.s);
+            dst_fin->send_sock = si;
+        }
+    }
+
+    if (dst_fin->send_sock == 0) {
+        dst_fin->send_sock=get_send_socket(0, &dst_fin->to, dst_fin->proto);
+        if (dst_fin->send_sock == 0) {
+            LM_ERR("can't forward to af %d, proto %d no corresponding"
+                    " listening socket\n", dst_fin->to.s.sa_family, dst_fin->proto);
+            goto error;
+        }
+    }
 
 	/* Version && proto && length */
 	hdr.hp_l = sizeof(struct hep_hdr);
@@ -1649,14 +1755,16 @@ static int trace_send_hep_duplicate(str *body, str *from, str *to)
 	memcpy((void*)(buffer + buflen) , (void*)body->s, body->len);
 	buflen +=body->len;
 
-	if (msg_send(&dst, buffer, buflen)<0)
+	if (msg_send(dst_fin, buffer, buflen)<0)
 	{
 		LM_ERR("cannot send hep duplicate message\n");
 		goto error;
 	}
 
+    if (p) {
 	free_proxy(p); /* frees only p content, not p itself */
 	pkg_free(p);
+    }
 	pkg_free(buffer);
 	return 0;
 error:
@@ -1779,16 +1887,16 @@ static void siptrace_rpc_status (rpc_t* rpc, void* c) {
 
 	if (strncasecmp(status.s, "on", strlen("on")) == 0) {
 		*trace_on_flag = 1;
-		rpc->printf(c, "Enabled");
+		rpc->rpl_printf(c, "Enabled");
 		return;
 	}
 	if (strncasecmp(status.s, "off", strlen("off")) == 0) {
 		*trace_on_flag = 0;
-		rpc->printf(c, "Disabled");
+		rpc->rpl_printf(c, "Disabled");
 		return;
 	}
 	if (strncasecmp(status.s, "check", strlen("check")) == 0) {
-		rpc->printf(c, *trace_on_flag ? "Enabled" : "Disabled");
+		rpc->rpl_printf(c, *trace_on_flag ? "Enabled" : "Disabled");
 		return;
 	} 
 	rpc->fault(c, 500, "Bad parameter (on, off or check)");
