@@ -23,7 +23,7 @@
  *
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software 
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 /*
  * History:
@@ -150,12 +150,12 @@ unsigned int get_on_branch(void)
 	return goto_on_branch;
 }
 
+void set_branch_route( unsigned int on_branch)
+{
+	branch_route = on_branch;
+}
 
 
-/* prepare_new_uac flags */
-#define UAC_DNS_FAILOVER_F 1 /**< new branch due to dns failover */
-#define UAC_SKIP_BR_DST_F  2 /**< don't set next hop as dst_uri for
-							   branch_route */
 
 
 /** prepares a new branch "buffer".
@@ -797,7 +797,7 @@ int add_blind_uac( /*struct cell *t*/ )
  *                    @see prepare_new_uac().
  *  @returns branch id (>=0) or error (<0)
 */
-static int add_uac( struct cell *t, struct sip_msg *request, str *uri,
+int add_uac( struct cell *t, struct sip_msg *request, str *uri,
 					str* next_hop, str* path, struct proxy_l *proxy,
 					struct socket_info* fsocket, snd_flags_t snd_flags,
 					int proto, int flags, str *instance, str *ruid,
@@ -1664,11 +1664,10 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 	/* make -Wall happy */
 	current_uri.s=0;
 
-	if (t->flags & T_CANCELED){
-		DBG("t_forward_non_ack: no forwarding on a canceled transaction\n");
-		ser_error=E_CANCELED;
-		return -1;
-	}
+	getbflagsval(0, &backup_bflags);
+
+	if (t->flags & T_CANCELED) goto canceled;
+
 	if (p_msg->REQ_METHOD==METHOD_CANCEL) { 
 		t_invite=t_lookupOriginalT(  p_msg );
 		if (t_invite!=T_NULL_CELL) {
@@ -1681,8 +1680,6 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 		}
 	}
 
-	getbflagsval(0, &backup_bflags);
-
 	/* if no more specific error code is known, use this */
 	lowest_ret=E_UNSPEC;
 	/* branches added */
@@ -1693,6 +1690,10 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 	if (t->on_branch) {
 		/* tell add_uac that it should run branch route actions */
 		branch_route = t->on_branch;
+		/* save the branch route so that it
+		 * can be used for adding branches later
+		 */
+		t->on_branch_delayed = t->on_branch;
 		/* reset the flag before running the actions (so that it
 		 * could be set again in branch_route if needed
 		 */
@@ -1723,6 +1724,8 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 							p_msg->fwd_send_flags, proto,
 							(p_msg->dst_uri.len)?0:UAC_SKIP_BR_DST_F, &p_msg->instance,
 							&p_msg->ruid, &p_msg->location_ua);
+		/* test if cancel was received meanwhile */
+		if (t->flags & T_CANCELED) goto canceled;
 		if (branch_ret>=0) 
 			added_branches |= 1<<branch_ret;
 		else
@@ -1740,6 +1743,8 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 							&path, proxy, si, p_msg->fwd_send_flags,
 							proto, (dst_uri.len)?0:UAC_SKIP_BR_DST_F, &instance,
 							&ruid, &location_ua);
+		/* test if cancel was received meanwhile */
+		if (t->flags & T_CANCELED) goto canceled;
 		/* pick some of the errors in case things go wrong;
 		   note that picking lowest error is just as good as
 		   any other algorithm which picks any other negative
@@ -1813,6 +1818,17 @@ int t_forward_nonack( struct cell *t, struct sip_msg* p_msg ,
 	ser_error=0; /* clear branch send errors, we have overall success */
 	set_kr(REQ_FWDED);
 	return 1;
+
+canceled:
+	DBG("t_forward_non_ack: no forwarding on a canceled transaction\n");
+	/* reset processed branches */
+	clear_branches();
+	/* restore backup flags from initial env */
+	setbflagsval(0, backup_bflags);
+	/* update message flags, if changed in branch route */
+	t->uas.request->flags = p_msg->flags;
+	ser_error=E_CANCELED;
+	return -1;
 }
 
 
